# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2004 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import GemRB
from GUIDefines import *
from ie_stats import *
from ie_spells import LS_MEMO
import GameCheck
import GUICommon
import Spellbook
import CommonTables

# HLA selection
HLAWindow = 0		# << HLA selection window
HLAAbilities = []	# << all learnable HLA abilities
HLANewAbilities = []	# << selected HLA abilites
HLADoneButton = 0	# << done button
HLATextArea = 0		# << HLA ability description area
HLACount = 0		# << number of HLA selections left
pc = 0			# << the pc
NumClasses = 0		# << number of classes
Classes = []		# << classes (ids)
Level = []		# << levels for each class
EnhanceGUI = 0		# << toggle for scrollbar and 25th hla slot

# setup our scroll index
GemRB.SetVar("HLATopIndex", 0)

def OpenHLAWindow (actor, numclasses, classes, levels):
	"""Opens the HLA selection window."""

	global HLAWindow, HLADoneButton, HLATextArea, HLACount, NumClasses, pc, Classes, Level
	global EnhanceGUI

	#enhance GUI?
	if (GemRB.GetVar("GUIEnhancements")&GE_SCROLLBARS):
		EnhanceGUI = 1

	# save our variables
	pc = actor
	NumClasses = numclasses
	Classes = classes
	Level = levels
	HLACount = GemRB.GetVar ("HLACount")

	# we use the same window as sorcerer spell selection
	HLAWindow = GemRB.LoadWindow (8, "GUIREC")

	# get all our HLAs (stored in HLAAbilities)
	GetHLAs ()

	# change the title to ABILITIES
	TitleLabel = HLAWindow.GetControl (0x10000017)
	TitleLabel.SetText (63818)

	# create the done button
	if GameCheck.IsBG2EE ():
		HLADoneButton = HLAWindow.GetControl (42)
	else:
		HLADoneButton = HLAWindow.GetControl (28)
	HLADoneButton.OnPress (HLADonePress)
	HLADoneButton.SetText(11973)
	HLADoneButton.MakeDefault()
	if HLACount:
		HLADoneButton.SetDisabled(True)
	else:
		HLADoneButton.SetDisabled(False)

	# setup our text area
	if GameCheck.IsBG2EE ():
		HLATextArea = HLAWindow.GetControl (41)
	else:
		HLATextArea = HLAWindow.GetControl (26)

	print("Number of HLAs:",len (HLAAbilities))

	# create a scrollbar if need-be
	if ( len (HLAAbilities) >= 25 ) and EnhanceGUI:
		# setup extra 25th HLA slot:
		HLAWindow.CreateButton (24, 231, 345, 42, 42)
		if ( len (HLAAbilities) > 25):
			# setup scrollbar
			ScrollBar = HLAWindow.CreateScrollBar (1000, {'x' : 290, 'y' : 142, 'w' : 16, 'h' : 252}, "GUISCRCW")
			ScrollBar.OnChange (HLAShowAbilities)
			#with enhanced GUI we have 5 rows of 5 abilities (the last one is 'the extra slot')
			count = GUICommon.ceildiv (len(HLAAbilities) - 25, 5) + 1
			ScrollBar.SetVarAssoc ("HLATopIndex", count, 0, count)
			HLAWindow.SetEventProxy(ScrollBar)

	# draw our HLAs and show the window
	HLAShowAbilities ()
	HLAWindow.ShowModal (MODAL_SHADOW_GRAY)

	return

def HLADonePress ():
	"""Saves the new HLAs and closes the HLA selection window."""

	# save all of our HLAs
	for i in range (len (HLANewAbilities)):
		# see if we're going to learn this ability
		if HLANewAbilities[i] == 0:
			continue

		# figure out the ability type
		HLARef = HLAAbilities[i][0]
		HLAType = HLARef[5:7]
		if HLAType == "PR":
			HLAType = IE_SPELL_TYPE_PRIEST
			HLALevel = int(HLARef[7])-1
		elif HLAType == "WI":
			HLAType = IE_SPELL_TYPE_WIZARD
			HLALevel = int(HLARef[7])-1
		else:
			HLAType = IE_SPELL_TYPE_INNATE
			HLALevel = 0

		# do we need to apply or learn it?
		if HLARef[:2] == "AP":
			GemRB.ApplySpell(pc, HLARef[3:])
		elif HLARef[:2] == "GA":
			# learn it or memorize another one
			Spellbook.LearnSpell (pc, HLARef[3:], HLAType, HLALevel, 1, LS_MEMO)

		#save the number of this HLA memorized
		GemRB.ApplyEffect(pc, "HLA", HLAAbilities[i][2], 0, HLARef[3:])

	# close the window
	if HLAWindow:
		HLAWindow.Close ()

	# so redraw skills knows we're done
	GemRB.SetVar ("HLACount", 0)

	return

def HLAShowAbilities ():
	"""Updates the HLA selections window.

	Called whenever an HLA is pressed."""

	j = ( GemRB.GetVar("HLATopIndex") + 1 ) * 5 - 5

	# we have a grid of 24 abilites
	for i in range (24+EnhanceGUI):
		# ensure we can learn this many abilites
		if len (HLAAbilities) < 25 and i == 24: #break if we don't need extra 25th button
			break
		SpellButton = HLAWindow.GetControl (i)
		if i + j >= len (HLAAbilities):
			SpellButton.SetDisabled(True)
			SpellButton.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
			continue
		else:
			SpellButton.SetDisabled(False)
			SpellButton.SetFlags(IE_GUI_BUTTON_NO_IMAGE, OP_NAND)

		# fill in the button with the spell data
		HLARef = HLAAbilities[i+j][0][3:]
		if not HLARef:
			continue
		Spell = GemRB.GetSpell (HLARef)
		SpellButton.SetTooltip(Spell['SpellName'])
		SpellButton.SetSpellIcon(HLARef, 1)
		SpellButton.SetVarAssoc("ButtonPressed", i)
		SpellButton.OnPress (HLASelectPress)
		SpellButton.SetSprites("GUIBTBUT", 0,0,1,2,3)
		SpellButton.SetFlags(IE_GUI_BUTTON_PICTURE, OP_OR)

		# don't allow the selection of an un-learnable ability
		if HLAAbilities[i+j][1] == 0:
			SpellButton.SetState(IE_GUI_BUTTON_LOCKED)
			# shade red
			color = {'r' : 200, 'g' : 0, 'b' : 0, 'a' : 100}
			SpellButton.SetBorder (0, color, 1,1)
		else:
			SpellButton.SetState (IE_GUI_BUTTON_ENABLED)
			# unset any borders on this button or an un-learnable from last level
			# will still shade red even though it is clickable
			SpellButton.SetBorder (0, None, 0,0)

	# show which spells are selected
	HLAShowSelectedAbilities ()

	GemRB.SetToken("number", str(HLACount))
	HLATextArea.SetText(63817)

	# show the points left
	PointsLeftLabel = HLAWindow.GetControl (0x10000018)
	PointsLeftLabel.SetText (str (HLACount))

	return

def HLASelectPress ():
	"""Toggles the HLA and displays a description string."""

	global HLACount, HLAAbilities, HLANewAbilities

	# get our variables
	j = ( GemRB.GetVar("HLATopIndex") + 1 ) * 5 - 5
	i = GemRB.GetVar ("ButtonPressed") + j

	# get the spell that's been pushed
	Spell = GemRB.GetSpell (HLAAbilities[i][0][3:])
	HLATextArea.SetText (Spell["SpellDesc"])

	# make sure we can learn the spell
	if HLAAbilities[i][1]:
		if HLANewAbilities[i]: # already picked -- unselecting
			# make we aren't the pre-req to another spell that is selected
			for j in range (len (HLAAbilities)):
				if (HLAAbilities[j][3] == HLAAbilities[i][0]) and (HLANewAbilities[j]):
					HLAShowSelectedAbilities () # so our pre-req is still highlighted
					return

			HLACount += 1
			HLANewAbilities[i] = 0
			HLAAbilities[i][2] -= 1 # internal counter
			HLADoneButton.SetDisabled (True)
		else: # selecting
			# we don't have any picks left
			if HLACount == 0:
				HLAMarkButton (i, 0)
				return

			# select the spell and change the done state if need be
			HLACount -= 1
			HLANewAbilities[i] = 1
			HLAAbilities[i][2] += 1 # increment internal counter
			if HLACount == 0:
				HLADoneButton.SetDisabled (False)

		# recheck internal exclusions and prereqs
		HLARecheckPrereqs (i)

	# show selected spells
	HLAShowAbilities ()
	HLAShowSelectedAbilities ()
	HLATextArea.SetText (Spell["SpellDesc"])

	# show the points left
	PointsLeftLabel = HLAWindow.GetControl (0x10000018)
	PointsLeftLabel.SetText (str (HLACount))
	return

def HLAShowSelectedAbilities ():
	"""Marks all of the selected abilities."""

	j = ( GemRB.GetVar("HLATopIndex") + 1 ) * 5 - 5

	# mark all of the abilities picked thus far
	for i in range (24+EnhanceGUI):
		if i + j >= len (HLANewAbilities): # make sure we don't call unavailable indexes
			break
		if HLANewAbilities[i+j]:
			HLAMarkButton (i+j, 1)
		else:
			HLAMarkButton (i+j, 0)

	return

def HLAMarkButton (i, select):
	"""Enables, disables, or highlights the given button.

	If select is true, the button is highlighted."""

	j = ( GemRB.GetVar("HLATopIndex") + 1 ) * 5 - 5

	if select:
		state = IE_GUI_BUTTON_SELECTED
	else:
		if HLAAbilities[i][1]:
			state = IE_GUI_BUTTON_ENABLED
		else: # can't learn
			state = IE_GUI_BUTTON_LOCKED

	# we have to use the index on the actual grid
	SpellButton = HLAWindow.GetControl(i-j)
	SpellButton.SetState(state)
	return

def GetHLAs ():
	"""Updates HLAAbilites with all the choosable class HLAs.

	HLAAbilities[x][0] is the given HLAs spell reference.
	HLAAbilities[x][1] is true if the HLAs prerequisites have been met."""

	global HLAAbilities, HLANewAbilities, HLACount

	# get some needed values
	Kit = GUICommon.GetKitIndex (pc)
	IsDual = GUICommon.IsDualClassed (pc, 0)
	IsDual = IsDual[0] > 0
	MaxHLACount = 0

	# reset the abilities
	HLAAbilities = []
	HLANewAbilities = []

	# the HLA table lookup table
	HLAAbbrTable = GemRB.LoadTable ("luabbr")

	# get all the HLAs for each class
	for i in range (NumClasses):
		ClassName = GUICommon.GetClassRowName (Classes[i], "class")
		CurrentLevel = Level[i]

		if Kit != 0 and NumClasses == 1 and not IsDual: # kitted single-class
			KitName = CommonTables.KitList.GetValue (Kit, 0)
			HLAClassTable = "lu" + HLAAbbrTable.GetValue (KitName, "ABBREV")
			ClassName = KitName
		else: # everyone else
			HLAClassTable = "lu" + HLAAbbrTable.GetValue (ClassName, "ABBREV")

		# actually load the table
		HLAClassTable = GemRB.LoadTable (HLAClassTable)
		print("HLA Class/Kit:", ClassName)

		# save all our HLAs from this class
		for j in range (HLAClassTable.GetRowCount ()):
			HLARef = HLAClassTable.GetValue (j, 0, GTV_STR)
			print("\tHLA " + str(j) +  ": " + HLARef)

			# make sure we have an ability here
			if HLARef == "*":
				print("\t\tEnd of HLAs")
				break

			# [ref to hla, memorizable?, num memorized, pre-req ref, excluded ref]
			SaveArray = [\
				HLARef,\
				0,\
				GemRB.CountEffects (pc, "HLA", -1, -1, HLARef[3:]),\
				HLAClassTable.GetValue (j, 6, GTV_STR),\
				HLAClassTable.GetValue (j, 7, GTV_STR)]

			# make sure we fall within the min and max parameters
			if HLAClassTable.GetValue (j, 3) > CurrentLevel or HLAClassTable.GetValue (j, 4) < CurrentLevel:
				print("\t\tNot within parameters")
				HLAAbilities.append(SaveArray)
				continue

			# see if we're alignment restricted (we never get them)
			HLAAlign = HLAClassTable.GetValue (j, 8, GTV_STR)
			if HLAAlign == "ALL_EVIL" and GemRB.GetPlayerStat (pc, IE_ALIGNMENT) < 6:
				# don't even save this one because we can never get it
				print("\t\tNeeds ALL_EVIL")
				continue
			elif HLAAlign == "ALL_GOOD" and GemRB.GetPlayerStat (pc, IE_ALIGNMENT) > 2:
				# ditto
				print("\t\tNeeds ALL_GOOD")
				continue

			# make sure we haven't already surpassed the number of time memorizable
			HLANumAllowed = HLAClassTable.GetValue (j, 5)
			print("\t\tHLA count: " + str(SaveArray[2]))
			if SaveArray[2] >= HLANumAllowed:
				print("\t\tOnly allowed to learn " + str(HLANumAllowed) + " times")
				HLAAbilities.append(SaveArray)
				continue

			# make sure we haven't learned an HLA that excludes this one
			HLAMemorized = GemRB.CountEffects (pc, "HLA", -1, -1, SaveArray[4][3:])
			print("\t\tHLAExcluded count: " + str(HLAMemorized))
			if (SaveArray[4] != "*") and (HLAMemorized > 0):
				print("\t\tExcluded by: " + str(SaveArray[4]))
				HLAAbilities.append(SaveArray)
				continue

			# we meet the prereqs so we can learn the HLA
			HLAMemorized = GemRB.CountEffects (pc, "HLA", -1, -1, SaveArray[3][3:])
			print("\t\tHLAPre count: " + str(HLAMemorized))
			if (SaveArray[3] == "*") or (HLAMemorized > 0):
				print("\t\tWe can learn it!")
				MaxHLACount += 1
				SaveArray[1] = 1
				HLAAbilities.append (SaveArray)
				continue

			# we didn't meet prereqs :(
			print("\t\tNeed pre-req: " + str(SaveArray[3]))
			HLAAbilities.append (SaveArray)

	# create an array to store our abilities as they are selected
	HLANewAbilities = [0]*len (HLAAbilities)

	#make sure we don't get stuck with HLAs we can't apply
	if MaxHLACount < HLACount:
		HLACount = MaxHLACount
		GemRB.SetVar ("HLACount", HLACount)

	return

def HLARecheckPrereqs (index):
	"""Rechecks the HLA prerequisites on the index on the fly."""

	# the number of times memorized
	Ref = HLAAbilities[index][0]
	Memorized = HLAAbilities[index][2]

	# check for new exclusions and pre-reqs
	for i in range (len (HLAAbilities)):
		# we don't need to check the index
		# this also fixes the assassination bug (it is excluded by itself)
		if i == index:
			continue
		# check for exclusions first
		if HLAAbilities[i][4] == Ref:
			if Memorized > 0: # can't learn it
				HLAAbilities[i][1] = 0
			else: # can, if it meets pre-reqs
				if HLAAbilities[i][3] != "*": # check prereqs
					for j in range (len (HLAAbilities)): # search for the prereq ref
						if (HLAAbilities[j][0] == HLAAbilities[i][3]) and (HLAAbilities[j][2] > 0): # can learn
							HLAAbilities[i][1] = 1
							break
				else: # no prereqs
					HLAAbilities[i][1] = 1

		# check for prereqs
		if HLAAbilities[i][3] == Ref:
			if Memorized > 0: # can learn if not excluded
				if HLAAbilities[i][4] != "*": # check for exclusions
					for j in range (len (HLAAbilities)): # search for the exclusion ref
						if (HLAAbilities[j][0] == HLAAbilities[i][4]) and (HLAAbilities[j][2] <= 0): # can learn
							HLAAbilities[i][1] = 1
							break
				else: # no exclusions
					HLAAbilities[i][1] = 1
			else: # prereqs not met
				HLAAbilities[i][1] = 0

	return





# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2007 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#character generation, class kit (GUICG22)

import GemRB
import CommonTables
from ie_stats import *
from GUIDefines import *

import CharGenCommon

KitWindow = 0
TextAreaControl = 0
DoneButton = 0
SchoolList = 0
ClassName = 0
TopIndex = 0
RowCount = 10
KitTable = 0
Init = 0
MyChar = 0
KitSelected = 0 #store clicked kit on redraw as number within RowCount
EnhanceGUI = GemRB.GetVar("GUIEnhancements")&GE_SCROLLBARS #extra kit button and scroll bar toggle

def OnLoad():
	global KitWindow, TextAreaControl, DoneButton
	global SchoolList, ClassName
	global RowCount, TopIndex, KitTable, Init, MyChar

	MyChar = GemRB.GetVar ("Slot")
	Race = GemRB.GetPlayerStat (MyChar, IE_RACE)
	RaceName = CommonTables.Races.GetRowName(CommonTables.Races.FindValue (3, Race) )

	ClassName = CommonTables.Classes.GetRowName (GemRB.GetPlayerStat (MyChar, IE_HITPOINTS)) # barbarian hack

	KitTable = GemRB.LoadTable("kittable")
	KitTableName = KitTable.GetValue(ClassName, RaceName)
	KitTable = GemRB.LoadTable(KitTableName,1)

	SchoolList = GemRB.LoadTable("magesch")

	#there is a specialist mage window, but it is easier to use
	#the class kit window for both
	KitWindow = GemRB.LoadWindow(22, "GUICG")
	CharGenCommon.PositionCharGenWin(KitWindow)
	
	if ClassName == "MAGE":
		Label = KitWindow.GetControl(0xfffffff)
		Label.SetText(595)

	for i in range(10):
		if i<4:
			Button = KitWindow.GetControl(i+1)
		else:
			Button = KitWindow.GetControl(i+5)
		Button.SetState(IE_GUI_BUTTON_DISABLED)
		Button.SetFlags(IE_GUI_BUTTON_RADIOBUTTON, OP_OR)

	if not KitTable: # sorcerer or monk
		RowCount = 1
	else:
		if ClassName == "MAGE": # mages
			RowCount = SchoolList.GetRowCount()
		else:
			RowCount = KitTable.GetRowCount()

	TopIndex = 0
	GemRB.SetVar("TopIndex", 0)
	if EnhanceGUI:
		tmpRowCount = RowCount
		if RowCount>10: #create 11 kit button
			extrakit = KitWindow.CreateButton (15, 18, 250, 271, 20)
			extrakit.SetState(IE_GUI_BUTTON_DISABLED)
			extrakit.SetFlags(IE_GUI_BUTTON_RADIOBUTTON|IE_GUI_BUTTON_CAPS, OP_OR)
			extrakit.SetSprites("GUICGBC",0, 0,1,2,3)
			RowCount = 11

		if tmpRowCount>11: #create scroll bar
			ScrollBar = KitWindow.CreateScrollBar(1000, {'x' : 290, 'y' : 50, 'w' : 16, 'h' : 220}, "GUISCRCW")
			ScrollBar.SetVarAssoc ("TopIndex", tmpRowCount-10, 0, tmpRowCount-10)
			ScrollBar.OnChange (RedrawKits)
			KitWindow.SetEventProxy(ScrollBar)

	elif not EnhanceGUI and RowCount>10:
		RowCount = 10

	for i in range(RowCount):
		if i<4:
			Button = KitWindow.GetControl(i+1)
		else:
			Button = KitWindow.GetControl(i+5)
		Button.SetVarAssoc("ButtonPressed", i)
		Button.OnPress (KitPress)

	BackButton = KitWindow.GetControl(8)
	BackButton.SetText(15416)
	BackButton.MakeEscape()
	DoneButton = KitWindow.GetControl(7)
	DoneButton.SetText(11973)
	DoneButton.MakeDefault()

	TextAreaControl = KitWindow.GetControl(5)
	TextAreaControl.SetText(17247)

	DoneButton.OnPress (NextPress)
	BackButton.OnPress (BackPress)
	Init = 1
	RedrawKits()
	KitPress()
	KitWindow.Focus()
	return

def RedrawKits():
	global TopIndex, Init, KitSelected

	TopIndex=GemRB.GetVar("TopIndex")
	EnabledButtons = []
	for i in range(RowCount):
		if i<4:
			Button = KitWindow.GetControl(i+1)
		else:
			Button = KitWindow.GetControl(i+5)
		Button.SetState(IE_GUI_BUTTON_DISABLED)
		if not KitTable:
			KitIndex = 0
			KitName = CommonTables.Classes.GetValue (ClassName, "NAME_REF")
		else:
			KitIndex = KitTable.GetValue (i + TopIndex, 0)
			if ClassName == "MAGE":
				KitName = SchoolList.GetValue (i+TopIndex, 0)
				if KitIndex == 0:
					KitName = SchoolList.GetValue ("GENERALIST", "NAME_REF")
					Button.SetState(IE_GUI_BUTTON_ENABLED)
					if Init: #preselection of mage plain kit
						Button.SetState(IE_GUI_BUTTON_SELECTED)
						KitSelected = i+TopIndex
						Init=0
				if KitIndex != "*":
					EnabledButtons.append (KitIndex - 21)
			else:
				if KitIndex and KitIndex != "*":
					KitName = CommonTables.KitList.GetValue (KitIndex, 1)
				else:
					KitName = CommonTables.Classes.GetValue (ClassName, "NAME_REF")
		Button.SetText(KitName)
		if not EnabledButtons or i+TopIndex in EnabledButtons:
			Button.SetState(IE_GUI_BUTTON_ENABLED)
			if Init and i+TopIndex>0:
				Button.SetState(IE_GUI_BUTTON_SELECTED)
				KitSelected = i+TopIndex
				Init=0
		if KitIndex == "*":
			continue
		if Init and i+TopIndex==0:
			if EnabledButtons:
				GemRB.SetVar("ButtonPressed", EnabledButtons[0]) #but leave Init==1
			else:
				GemRB.SetVar("ButtonPressed", 0)
				Button.SetState(IE_GUI_BUTTON_SELECTED) 
				KitSelected = i+TopIndex
				Init=0
		if not Init and i+TopIndex == KitSelected: #remark selection state on redraw
			Button.SetState(IE_GUI_BUTTON_SELECTED)
	return

def KitPress():
	global KitSelected

	ButtonPressed=GemRB.GetVar("ButtonPressed")
	KitSelected = ButtonPressed + TopIndex
	if not KitTable:
		KitIndex = 0
	else:
		KitIndex = KitTable.GetValue (ButtonPressed + TopIndex, 0)
		if ClassName == "MAGE":
			if ButtonPressed + TopIndex == 0:
				KitIndex = 0
			else:
				KitIndex = ButtonPressed + TopIndex + 21

	if ClassName == "MAGE" and KitIndex != 0:
		GemRB.SetVar ("MAGESCHOOL", KitIndex - 21) # hack: -21 to make the generalist 0
	else:
		GemRB.SetVar("MAGESCHOOL", 0) # so bards don't get schools

	if KitIndex == 0:
		KitDescription = CommonTables.Classes.GetValue (ClassName, "DESC_REF")
	else:
		KitDescription = CommonTables.KitList.GetValue (KitIndex, 3)

	TextAreaControl.SetText(KitDescription)
	DoneButton.SetState(IE_GUI_BUTTON_ENABLED)

	GemRB.SetVar ("Class Kit", KitIndex)

	return

def BackPress():
	GemRB.SetVar("Class Kit", 0) # reverting the value so we are idempotent
	GemRB.SetVar("MAGESCHOOL", 0)
	if KitWindow:
		KitWindow.Close ()
	GemRB.SetNextScript("GUICG2")
	return

def NextPress():
	if KitWindow:
		KitWindow.Close ()

	#make gnomes always kitted
	KitIndex = GemRB.GetVar ("Class Kit")
	MageSchool = GemRB.GetVar ("MAGESCHOOL")
	if MageSchool and not KitIndex:
		KitValue = SchoolList.GetValue (MageSchool, 3)
	else:
		KitValue = CommonTables.KitList.GetValue (KitIndex, 6)

	#save the kit
	GemRB.SetPlayerStat (MyChar, IE_KIT, KitValue)

	GemRB.SetNextScript("CharGen4") #abilities
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2010 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#################### Interchapter text screen functions #################

import GemRB
from ie_restype import RES_2DA
from ie_sounds import CHAN_GUI, SND_SPEECH, GEM_SND_VOL_AMBIENTS
from GUIDefines import *
import GameCheck

TextScreen = None
TextArea = None
Row = 1
TableName = None

def FindTextRow (Table):
	global Row

	#this is still not the full implementation, but the original engine never used this stuff
	Row = Table.GetRowIndex("DEFAULT")
	if Table.GetValue (Row, 1)== -1:
		if GemRB.GameGetReputation() >= 100:
			Row = Table.GetRowIndex("GOOD_REPUTATION")
		else:
			Row = Table.GetRowIndex("BAD_REPUTATION")
	return

def StartTextScreen ():
	global TextScreen, TextArea, TableName, Row

	# for easier development
	if GameCheck.IsGemRBDemo () and GemRB.GetVar ("SkipIntroVideos") and not GemRB.GetGameVar ("completed"):
		GemRB.GamePause (0, 3)
		return

	GemRB.GamePause (1, 3)
	ToggleAmbients (0)

	ID = -1
	MusicName = "*"
	Message = 17556   # default: Paused for chapter text
	TableName = GemRB.GetGameString (STR_TEXTSCREEN)

	#iwd2/bg2 has no separate music
	if GameCheck.IsIWD1():
		if TableName == "":
			MusicName = "chap0"
		else:
			MusicName = "chap1"
		TableName = "chapters"
	elif GameCheck.IsIWD2():
		TableName = "chapters"
	elif GameCheck.IsGemRBDemo ():
		#TODO: set MusicName
		TableName = "chapters"
		Message = "*"

	if TableName == "":
		EndTextScreen ()
		return

	if GemRB.HasResource ("textscrn", RES_2DA, 1):
		TextTable = GemRB.LoadTable ("textscrn", 1)
		if TextTable != None:
			TxtRow = TextTable.GetRowIndex (TableName)
			if TxtRow is not None:
				ID = TextTable.GetValue (TxtRow, 0)
				MusicName = TextTable.GetValue (TxtRow, 1)
				Message = TextTable.GetValue (TxtRow, 2)

	if Message != "*":
		GemRB.DisplayString (Message, ColorRed)

	Table = GemRB.LoadTable (TableName)
	if GameCheck.IsBG2OrEE ():
		LoadPic = Table.GetValue (-1, -1)
		if LoadPic.startswith ("*"): # BG2 epilogues
			ID = 63
			LoadPic = LoadPic.replace ("*", "")
		else:
			ID = 62
	elif ID == -1:
		#default: try to determine ID from current chapter
		ID = GemRB.GetGameVar("CHAPTER") & 0x7fffffff
		Chapter = ID + 1

	# Textscreen should always stop currently playing music before starting
	# to play any chapter music.
	GemRB.HardEndPL ()

	if MusicName != "*":
		GemRB.LoadMusicPL (MusicName + ".mus")

	TextScreen = GemRB.LoadWindow (ID, "GUICHAP")
	TextArea = TextScreen.GetControl (2)
	TextArea.SetFlags (IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
	TextArea.SetColor (ColorWhitish, TA_COLOR_INITIALS)

	GameWin = GemRB.GetView("GAMEWIN")
	GameWin.SetDisabled(True)

	if GameCheck.IsBG1():
		#these suckers couldn't use a fix row
		FindTextRow (Table)
	elif GameCheck.IsBG2OrEE ():
		FindTextRow (Table)
		if LoadPic != "":
			if ID == 63:
				#only for BG2 epilogue windows
				PicButton = TextScreen.GetControl (4)
				PicButton.SetPicture (LoadPic)
				PicButton.SetState (IE_GUI_BUTTON_LOCKED)
			else:
				TextScreen.SetBackground (LoadPic)
	else:
		Row = Chapter

	#caption
	Value = Table.GetValue (Row, 0)
	#don't display the fake -1 string (No caption in toscst.2da)
	Label = TextScreen.GetControl (0x10000000)
	if Label and Value!="NONE" and Value>0:
		Label.SetText (Value)

	#done
	Button=TextScreen.GetControl (0)
	if GameCheck.IsGemRBDemo ():
		Button.SetText (84)
	else:
		Button.SetText (11973)
	Button.OnPress (EndTextScreen)
	Button.MakeDefault()

	#replay
	Button=TextScreen.GetControl (3)
	if GameCheck.IsGemRBDemo ():
		Button.SetText (85)
	else:
		Button.SetText (16510)
	Button.OnPress (ReplayTextScreen)

	#if this was opened from somewhere other than game control close that window
	import GUICommonWindows
	GUICommonWindows.CloseTopWindow ()

	TextScreen.ShowModal (MODAL_SHADOW_NONE)

	ReplayTextScreen()
	return

# FIXME: move to GUIScript as a PauseAmbients method that works directly with the AmbientMgr
AmbientVolume = 0
def ToggleAmbients (state):
	global AmbientVolume

	if state:
		GemRB.SetVar ("Volume Ambients", AmbientVolume)
	else:
		AmbientVolume = GemRB.GetVar ("Volume Ambients")
		GemRB.SetVar ("Volume Ambients", 0)

	GemRB.UpdateVolume (GEM_SND_VOL_AMBIENTS)

def EndTextScreen ():
	global TextScreen, TableName

	if TextScreen:
		TextScreen.Close ()
		GemRB.HardEndPL ()
		GemRB.PlaySound(None, CHAN_GUI, 0, 0, SND_SPEECH)

	GameWin = GemRB.GetView("GAMEWIN")
	GameWin.SetDisabled(False)
	ToggleAmbients (1)
	GemRB.GamePause (0, 3)

	if TableName == "25ecred":
		GemRB.SetNextScript("QuitGame")
	return

def ReplayTextScreen ():
	global TextArea, TableName, Row

	GemRB.PlaySound(None, CHAN_GUI, 0, 0, SND_SPEECH)

	Table = GemRB.LoadTable (TableName)
	Count = Table.GetColumnCount (Row)

	#hack for messy chapters.2da in IWD
	#note: iwd doesn't use TextScreen, only IncrementChapter
	if GameCheck.IsIWD1() or GameCheck.IsIWD2():
		Count = 2

	text = ""
	for i in range(1, Count):
		# flag value of 14 = IE_STR_SOUND|IE_STR_SPEECH/GEM_SND_SPEECH|GEM_SND_QUEUE
		text += "\n\n" if not GameCheck.IsGemRBDemo () else ""
		text += GemRB.GetString(Table.GetValue (Row, i), 14)

	TextArea.SetChapterText (text)

	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2005 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# Start.py - intro and main menu screens

###################################################

import GemRB
import GameCheck
from GUIDefines import *
from ie_restype import *

StartWindow = None
JoinGameButton = 0

def OnLoad ():
	global JoinGameButton

	GemRB.SetVar("ExpansionGame", 0)

	skip_videos = GemRB.GetVar ("SkipIntroVideos")
	if not skip_videos:
		GemRB.PlayMovie ("BISLOGO", 1)
		if GameCheck.HasHOW():
			GemRB.PlayMovie ("WOTC", 1)
		else:
			GemRB.PlayMovie ("TSRLOGO", 1)
		GemRB.PlayMovie("INTRO", 1)
		GemRB.SetVar ("SkipIntroVideos", 1)

	if GameCheck.HasHOW():
		GemRB.SetMasterScript("BALDUR","WORLDMAP","EXPMAP")
	else:
		GemRB.SetMasterScript("BALDUR","WORLDMAP")

#main window
	global StartWindow
	StartWindow = GemRB.LoadWindow (0, "GUICONN")
	ProtocolButton = StartWindow.GetControl (0x00)
	CreateGameButton = StartWindow.GetControl (0x02)
	LoadGameButton = StartWindow.GetControl (0x07)
	JoinGameButton = StartWindow.GetControl (0x03)
	MoviesButton = StartWindow.GetControl (0x08)
	QuitGameButton = StartWindow.GetControl (0x01)
	VersionLabel = StartWindow.CreateLabel (0x0fff0000, 0, 0, 640, 30, "REALMS2", "", IE_FONT_SINGLE_LINE | IE_FONT_ALIGN_CENTER)
	VersionLabel.SetText (GemRB.Version)
	VersionLabel.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})
	ProtocolButton.SetStatus (IE_GUI_BUTTON_ENABLED)
	CreateGameButton.SetStatus (IE_GUI_BUTTON_ENABLED)
	LoadGameButton.SetStatus (IE_GUI_BUTTON_ENABLED)
	MoviesButton.SetStatus (IE_GUI_BUTTON_ENABLED)
	QuitGameButton.SetStatus (IE_GUI_BUTTON_ENABLED)
	LastProtocol = GemRB.GetVar ("Last Protocol Used")
	if LastProtocol == 0:
		ProtocolButton.SetText (15413)
		JoinGameButton.SetStatus (IE_GUI_BUTTON_DISABLED)
	elif LastProtocol == 1:
		ProtocolButton.SetText (13967)
		JoinGameButton.SetStatus (IE_GUI_BUTTON_ENABLED)
	elif LastProtocol == 2:
		ProtocolButton.SetText (13968)
		JoinGameButton.SetStatus (IE_GUI_BUTTON_ENABLED)
	CreateGameButton.SetText (13963)
	LoadGameButton.SetText (13729)
	JoinGameButton.SetText (13964)
	MoviesButton.SetText (15415)
	QuitGameButton.SetText (13731)
	QuitGameButton.OnPress (QuitPress)
	QuitGameButton.MakeEscape ()
	ProtocolButton.OnPress (ProtocolPress)
	MoviesButton.OnPress (MoviesPress)
	LoadGameButton.OnPress (LoadPress)
	CreateGameButton.OnPress (CreatePress)
	StartWindow.Focus ()
	GemRB.LoadMusicPL("Theme.mus",1)
	
	StartWindow.SetAction(RefreshProtocol, ACTION_WINDOW_FOCUS_GAINED)
	
	return

def ProtocolPress ():
	ProtocolWindow = GemRB.LoadWindow (1, "GUICONN")
	
	#Disabling Unused Buttons in this Window
	Button = ProtocolWindow.GetControl (2)
	Button.SetState (IE_GUI_BUTTON_DISABLED)
	Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
	Button = ProtocolWindow.GetControl (3)
	Button.SetState (IE_GUI_BUTTON_DISABLED)
	Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
	Button = ProtocolWindow.GetControl (9)
	Button.SetState (IE_GUI_BUTTON_DISABLED)
	Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
	
	SinglePlayerButton = ProtocolWindow.GetControl (10)
	SinglePlayerButton.SetFlags (IE_GUI_BUTTON_RADIOBUTTON,OP_OR)
	SinglePlayerButton.SetText (15413)
	
	IPXButton = ProtocolWindow.GetControl (0)
	IPXButton.SetFlags (IE_GUI_BUTTON_RADIOBUTTON,OP_OR)
	IPXButton.SetText (13967)
	
	TCPIPButton = ProtocolWindow.GetControl (1)
	TCPIPButton.SetFlags (IE_GUI_BUTTON_RADIOBUTTON,OP_OR)
	TCPIPButton.SetText (13968)
	
	SinglePlayerButton.SetVarAssoc ("Last Protocol Used", 0)
	IPXButton.SetVarAssoc ("Last Protocol Used", 1)
	TCPIPButton.SetVarAssoc ("Last Protocol Used", 2)
	
	TextArea = ProtocolWindow.GetControl (7)
	TextArea.SetText (11316)
	
	DoneButton = ProtocolWindow.GetControl (6)
	DoneButton.SetText (11973)
	DoneButton.OnPress (ProtocolWindow.Close)
	DoneButton.MakeEscape()
	
	ProtocolWindow.ShowModal (1)
	return
	
def RefreshProtocol (win):
	ProtocolButton = win.GetControl (0)
	
	LastProtocol = GemRB.GetVar ("Last Protocol Used")
	if LastProtocol == 0:
		ProtocolButton.SetText (15413)
		JoinGameButton.SetStatus (IE_GUI_BUTTON_DISABLED)
	elif LastProtocol == 1:
		ProtocolButton.SetText (13967)
		JoinGameButton.SetStatus (IE_GUI_BUTTON_ENABLED)
	elif LastProtocol == 2:
		ProtocolButton.SetText (13968)
		JoinGameButton.SetStatus (IE_GUI_BUTTON_ENABLED)

	return
	
def CreatePress ():
	global GameTypeWindow, ExpansionType

	if not GameCheck.HasHOW():
		GameTypeReallyDonePress()

	GameTypeWindow = GemRB.LoadWindow (24, "GUICONN")

	CancelButton = GameTypeWindow.GetControl (1)
	CancelButton.SetText (13727)
	CancelButton.OnPress (GameTypeWindow.Close)
	CancelButton.MakeEscape()

	DoneButton = GameTypeWindow.GetControl (2)
	DoneButton.SetText (11973)
	DoneButton.OnPress (lambda: GameTypeDonePress(GameTypeWindow))
	DoneButton.MakeDefault()

	FullGameButton = GameTypeWindow.GetControl (4)
	FullGameButton.SetFlags (IE_GUI_BUTTON_RADIOBUTTON,OP_OR)
	FullGameButton.OnPress (lambda btn: GameTypeChange (btn, False))
	FullGameButton.SetText (24869)

	ExpansionGameButton = GameTypeWindow.GetControl (5)
	ExpansionGameButton.SetFlags (IE_GUI_BUTTON_RADIOBUTTON,OP_OR)
	ExpansionGameButton.OnPress (lambda btn: GameTypeChange (btn, True))
	ExpansionGameButton.SetText (24871)

	ExpansionGameButton.SetVarAssoc ("ExpansionGame", 1)
	FullGameButton.SetVarAssoc ("ExpansionGame", 0)

	GameTypeChange (None, False)
	GameTypeWindow.ShowModal (1)
	return

def GameTypeChange(btn, isExpansion):
	GameTypeTextArea = GameTypeWindow.GetControl(3)
	if isExpansion:
		GameTypeTextArea.SetText (24872)
	else:
		GameTypeTextArea.SetText (24870)
	return

def GameTypeDonePress(win):
	#todo: importing team members from final save (string 26317)?

	ExpansionGame = GemRB.GetVar("ExpansionGame")
	if ExpansionGame == 0: #start in Easthaven
		win.Close()
		GameTypeReallyDonePress()
	elif ExpansionGame == 1: #show a warning message first
		GameType2Window = GemRB.LoadWindow (25, "GUICONN")

		TextArea = GameType2Window.GetControl(0)
		TextArea.SetText(26318)

		def confirm():
			win.Close()
			GameType2Window.Close()
			GameTypeReallyDonePress()

		YesButton = GameType2Window.GetControl (1)
		YesButton.SetText (13912)
		YesButton.OnPress (confirm)

		NoButton = GameType2Window.GetControl (2)
		NoButton.SetText (13913)
		NoButton.OnPress (GameType2Window.Close)
		NoButton.MakeEscape()

		def cancel():
			win.Close();
			GameType2Window.Close()

		CancelButton = GameType2Window.GetControl (3)
		CancelButton.SetText (13727)
		CancelButton.OnPress (cancel)
		CancelButton.MakeEscape()

		GameType2Window.ShowModal(1)

def GameTypeReallyDonePress():
	StartWindow.Close()
	GemRB.LoadGame(None)
	GemRB.SetNextScript ("PartyFormation")

def LoadPress ():
	GemRB.SetNextScript ("GUILOAD")
	return

def MoviesPress ():
	GemRB.SetNextScript ("GUIMOVIE")
	return
	
def QuitPress ():
	QuitWindow = GemRB.LoadWindow (22, "GUICONN")
	CancelButton = QuitWindow.GetControl (2)
	CancelButton.SetText (13727)
	CancelButton.OnPress (QuitWindow.Close)
	CancelButton.MakeEscape()

	QuitButton = QuitWindow.GetControl (1)
	QuitButton.SetText (15417)
	QuitButton.OnPress (lambda: GemRB.Quit())
	QuitButton.MakeDefault()

	TextArea = QuitWindow.GetControl (0)
	TextArea.SetText (19532)
	QuitWindow.ShowModal (1)
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import random

import GemRB
import GUICommonWindows
from GUICommon import BindControlCallbackParams
from GUIDefines import *
from ie_stats import *

def GetPortraitButtons(Window, *args):
	list = []
	for i in range(6):
		btn = Window.GetControl(i)
		btn.SetVarAssoc("portrait", i + 1)
		list.append(btn)
	return list

def PortraitButtonHPOnPress (btn):
	id = btn.ControlID - 6
	hbs = GemRB.GetVar('Health Bar Settings')
	GemRB.SetVar('Health Bar Settings', hbs ^ (1 << id))
	UpdatePortraitButton(btn.Window.GetControl(id))
	return

def SetupButtonBorders (Button):
	yellow = {'r' : 255, 'g' : 255, 'b' : 0, 'a' : 255}
	green = {'r' : 0, 'g' : 255, 'b' : 0, 'a' : 255}
	Button.SetBorder (0, green, 0, 0, Button.GetInsetFrame (1, 1, 2, 2))
	Button.SetBorder (1, yellow, 0, 0, Button.GetInsetFrame (3, 3, 4, 4))

def OpenPortraitWindow (pos=WINDOW_RIGHT|WINDOW_VCENTER):
	Window = GemRB.LoadWindow (1, "GUIWORLD", pos)
	Window.AddAlias("PORTWIN")
	Window.SetFlags(WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)

	# Rest
	Button = Window.GetControl (6)
	Button.SetTooltip (11942)
	Button.OnPress (GUICommonWindows.RestPress)

	PortraitButtons = GetPortraitButtons (Window)
	for Button in PortraitButtons:
		pcID = Button.Value

		Button.OnRightPress (GUICommonWindows.OpenInventoryWindowClick)
		Button.OnPress (GUICommonWindows.PortraitButtonOnPress)
		Button.SetAction (GUICommonWindows.PortraitButtonOnShiftPress, IE_ACT_MOUSE_PRESS, GEM_MB_ACTION, GEM_MOD_CTRL, 1)
		Button.SetAction (GUICommonWindows.ButtonDragSourceHandler, IE_ACT_DRAG_DROP_SRC)
		Button.SetAction (GUICommonWindows.ButtonDragDestHandler, IE_ACT_DRAG_DROP_DST)
		Button.SetAction(BindControlCallbackParams(GemRB.GameControlLocateActor, pcID), IE_ACT_MOUSE_ENTER)
		Button.SetAction(lambda: GemRB.GameControlLocateActor(-1), IE_ACT_MOUSE_LEAVE)
		Button.SetBAM ("PPPANN", 0, 0, -1) # NOTE: just a dummy, won't be visible
		Button.SetHotKey(chr(ord('0') + pcID), 0, True)
		Button.OnAnimEnd (UpdateButtonAnimation)
		SetupButtonBorders (Button)

		ButtonHP = Window.GetControl (5 + pcID)
		ButtonHP.SetFlags(IE_GUI_BUTTON_PICTURE, OP_SET)
		ButtonHP.SetBAM('FILLBAR', 0, 0, -1)
		ButtonHP.OnPress(PortraitButtonHPOnPress)

	UpdatePortraitWindow()
	return Window
	
def UpdatePortraitWindow ():
	"""Updates all of the portraits."""

	Window = GemRB.GetView("PORTWIN")

	pc = GemRB.GameGetSelectedPCSingle ()

	PortraitButtons = GetPortraitButtons (Window)
	for Button in PortraitButtons:
		pcID = Button.Value

		UpdatePortraitButton(Button)
		if GUICommonWindows.SelectionChangeHandler:
			Button.EnableBorder(0, pc == pcID)
		else:
			Button.EnableBorder(0, GemRB.GameIsPCSelected(pcID))

	return

def UpdateButtonAnimation(Button):
	"""Selects the correct portrait cycle depending on character state"""
	# note: there are actually two portraits per chr, eg PPPANN (static), WMPANN (animated)

	pcID = Button.ControlID + 1

	state = GemRB.GetPlayerStat (pcID, IE_STATE_ID)
	hp = GemRB.GetPlayerStat (pcID, IE_HITPOINTS)
	hp_max = GemRB.GetPlayerStat (pcID, IE_MAXHITPOINTS)

	if state & STATE_DEAD:
		cycle = 9
	elif state & STATE_HELPLESS:
		cycle = 8
	elif state & STATE_PETRIFIED:
		cycle = 7
	elif state & STATE_PANIC: # frozen?
		cycle = 6
	elif state & STATE_POISONED:
		cycle = 2 # and 3
	elif hp < hp_max / 2:
		cycle = 4 # and 5
	else:
		cycle = 0 # and 1

	portrait = GemRB.GetPlayerPortrait (pcID, 0)
	pic = portrait["ResRef"]

	if cycle < 6: # a "random" animation, other cycles are not animated
		def phase2(btn):
			switch = random.randint(0, 29) == 0
			btn.SetAnimation(pic, cycle + int(switch), A_ANI_PLAYONCE)
			btn.OnAnimEnd (UpdateButtonAnimation)
		# note: the granularity of time should be
		# one of twenty values from [500, 10000]
		# but not the full range.
		duration = 500 + 500 * random.randint(0, 19);
		Button.SetAnimation([GemRB.GetSprite(pic, -1, cycle, 0)], duration, A_ANI_PLAYONCE)
		Button.OnAnimEnd (phase2)
	else:
		Button.SetAnimation(pic, cycle, A_ANI_PLAYONCE)
		Button.OnAnimEnd (None)
	return

def UpdatePortraitButton(Button):
	pcID = Button.ControlID + 1
	portrait = GemRB.GetPlayerPortrait (pcID, 0)
	if not portrait:
		Button.SetVisible(False)
		Button.SetAnimation(None)
		return

	Button.SetVisible (True)

	hp = GemRB.GetPlayerStat (pcID, IE_HITPOINTS)
	hp_max = GemRB.GetPlayerStat (pcID, IE_MAXHITPOINTS)

	if hp_max < 1 or hp == "?":
		ratio = 0.0
	else:
		ratio = hp / float(hp_max)
		if ratio > 1.0: ratio = 1.0

	r = int (255 * (1.0 - ratio))
	g = int (255 * ratio)

	ButtonHP = Button.Window.GetControl(6 + Button.ControlID)

	ButtonHP.SetColor ({'r' : r, 'g' : g, 'b' : 0})
	ButtonHP.SetPictureClipping (ratio)

	if GemRB.GetVar('Health Bar Settings') & (1 << Button.ControlID):
		ButtonHP.SetFlags (IE_GUI_BUTTON_PICTURE, OP_OR)
		ButtonHP.SetText ("")
	else:
		ButtonHP.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)
		ButtonHP.SetText ("{} / {}".format(hp, hp_max))

	GemRB.GetVar ("{}_ANIM".format(Button.ControlID))

	UpdateButtonAnimation(Button)
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
#character generation, class kit (GUICG22)
import GemRB
from GUIDefines import *
from ie_stats import *
import CharGenCommon
import GUICommon
import CommonTables


KitWindow = 0
TextAreaControl = 0
DoneButton = 0
SchoolList = 0
ClassName = 0

def OnLoad():
	global KitWindow, TextAreaControl, DoneButton
	global SchoolList, ClassName

	RaceName = CommonTables.Races.GetRowName(GemRB.GetVar("Race")-1 )
	ClassName = GUICommon.GetClassRowName (GemRB.GetVar ("Class")-1, "index")
	KitTable = GemRB.LoadTable("kittable")
	KitTableName = KitTable.GetValue(ClassName, RaceName)
	KitTable = GemRB.LoadTable(KitTableName,1)

	SchoolList = GemRB.LoadTable("magesch")

	#there is only a specialist mage window for bg1
	KitWindow = GemRB.LoadWindow(12, "GUICG")

	for i in range(8):
		Button = KitWindow.GetControl(i+2)
		Button.SetState(IE_GUI_BUTTON_DISABLED)
		Button.SetFlags(IE_GUI_BUTTON_RADIOBUTTON, OP_OR)

	if not KitTable:
		RowCount = 1
	else:
		RowCount = KitTable.GetRowCount()

	for i in range(RowCount):
		Button = KitWindow.GetControl(i+2)
		if not KitTable:
			if ClassName == "MAGE":
				KitIndex = GemRB.GetVar ("MAGESCHOOL")
				KitName = SchoolList.GetValue(i, 0)
			else:
				KitIndex = 0
				KitName = CommonTables.Classes.GetValue(ClassName, "NAME_REF")

		else:
			KitIndex = KitTable.GetValue (i, 0)
			if ClassName == "MAGE":
				if KitIndex:
					KitIndex = KitIndex - 21
				KitName = SchoolList.GetValue (KitIndex, 0)
			else:
				if KitIndex:
					KitName = CommonTables.KitList.GetValue (KitIndex, 1)
				else:
					KitName = CommonTables.Classes.GetValue (ClassName, "NAME_REF")

		Button.SetState(IE_GUI_BUTTON_ENABLED)
		Button.SetText(KitName)
		Button.SetVarAssoc ("Class Kit", KitIndex)
		if i==0:
			GemRB.SetVar ("Class Kit", KitIndex)
		Button.OnPress (KitPress)

	BackButton = KitWindow.GetControl(12)
	BackButton.SetText(15416)
	DoneButton = KitWindow.GetControl(0)
	DoneButton.SetText(11973)
	DoneButton.MakeDefault()

	TextAreaControl = KitWindow.GetControl(11)
	TextAreaControl.SetText(17245)

	DoneButton.OnPress (NextPress)
	BackButton.OnPress (lambda: CharGenCommon.back(KitWindow))
	#KitPress()
	KitWindow.ShowModal(MODAL_SHADOW_GRAY)
	return

def KitPress():
	KitIndex = GemRB.GetVar ("Class Kit")
	if KitIndex == 0:
		KitName = CommonTables.Classes.GetValue(ClassName, "DESC_REF")
	else:
		if ClassName == "MAGE":
			KitName = SchoolList.GetValue (KitIndex, 1)
		else:
			KitName = CommonTables.KitList.GetValue (KitIndex, 3)
	TextAreaControl.SetText(KitName)
	DoneButton.SetState(IE_GUI_BUTTON_ENABLED)
	return

def NextPress():
	KitWindow.Close ()
	#class	
	Class = CommonTables.Classes.GetValue (ClassName, "ID")
	MyChar = GemRB.GetVar ("Slot")
	GemRB.SetPlayerStat (MyChar, IE_CLASS, Class)
	KitIndex = GemRB.GetVar ("Class Kit")
	if ClassName == "MAGE":
		GemRB.SetVar("MAGESCHOOL", KitIndex)
	#the same as the unusable field
	Kit = CommonTables.KitList.GetValue(KitIndex, 6)
	GemRB.SetPlayerStat (MyChar, IE_KIT, Kit)
	CharGenCommon.next()



# GemRB - Infinity Engine Emulator
# Copyright (C) 2012 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# GUIOPTControls.py - control functions for the GUIOPT winpack:

###################################################

import GemRB
import GameCheck
from GUIDefines import *

###################################################
# strrefs
if GameCheck.IsPST():
	STR_OPT_DONE = 1403
	STR_OPT_CANCEL = 4196
else:
	STR_OPT_DONE = 11973
	STR_OPT_CANCEL = 13727

###################################################
# These functions help to setup controls found
# in Video, Audio, Gameplay, Feedback and Autopause
# options windows

# These controls are usually made from an active
# control (button, slider ...) and a label

# NOTE: make sure handler users also set the strref in them!
def OptSlider (ctlhelp, slider_id, nameID, nameStrRef, variable, action = None, value = 1):
	"""Standard slider for option windows"""

	window = GetWindow ()
	slider = window.GetControl (slider_id)
	helpTA = GemRB.GetView ("OPTHELP")
	slider.SetVarAssoc (variable, value)
	if action:
		slider.OnChange (action)
	else:
		# create an anonymous callback, so we don't need to create a separate function for each string
		slider.OnChange (lambda: helpTA.SetText (ctlhelp))

	OptBuddyLabel (nameID, nameStrRef, ctlhelp, helpTA.Value)
	if HasMouseOver ():
		slider.OnMouseEnter (lambda: helpTA.SetText (ctlhelp))
		slider.OnMouseLeave (lambda: helpTA.SetText (helpTA.Value))

	return slider

def OptRadio (action, button_id, nameID, variable, value, nameStrRef = None, focusedText = None, defaultText = None):
	"""Standard radio button for option windows"""

	window = GetWindow ()
	button = window.GetControl (button_id)
	button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
	button.OnPress (action)
	button.SetVarAssoc (variable, value)
	if GameCheck.IsIWD2():
		button.SetSprites("GBTNOPT4", 0, 0, 1, 2, 3)
	elif GameCheck.IsIWD1() or GameCheck.IsBG1():
		button.SetSprites ("TOGGLE", 0, 0, 1, 3, 2)

	OptBuddyLabel (nameID, nameStrRef, focusedText, defaultText)

	return button

# NOTE: make sure handler users also set the strref in them!
def OptCheckbox (ctlhelp, button_id, nameID, nameStrRef, variable, handler = None, value = 1):
	"""Standard checkbox for option windows"""

	window = GetWindow ()
	button = window.GetControl (button_id)
	button.SetFlags (IE_GUI_BUTTON_CHECKBOX, OP_OR)
	if variable:
		button.SetVarAssoc (variable, value)

	if GameCheck.IsIWD2():
		button.SetSprites("GBTNOPT4", 0, 0, 1, 2, 3)
	elif GameCheck.IsIWD1() or GameCheck.IsBG1():
		button.SetSprites ("GMPPARBC", 3, 1, 2, 3, 5)

	def callback():
		GemRB.GetView ("OPTHELP").SetText (ctlhelp)

	if handler:
		button.OnPress (handler)
	elif HasMouseOver ():
		button.OnMouseEnter (callback)
	else:
		button.OnPress (callback)

	OptBuddyLabel (nameID, nameStrRef, ctlhelp, GemRB.GetView ("OPTHELP").Value)

	return button

def OptButton (action, button_id, button_strref):
	"""Standard subwindow button for option windows"""

	window = GetWindow ()
	button = window.GetControl (button_id)
	button.OnPress (action)
	button.SetText (button_strref)

def OptDone (action, button_id):
	"""Standard `Done' button for option windows"""

	window = GetWindow ()
	button = window.GetControl (button_id)
	button.SetText (STR_OPT_DONE) # Done
	button.OnPress (action)
	button.MakeDefault()

	if GameCheck.IsPST():
		button.SetVarAssoc ("Cancel", 0)

def OptCancel (action, button_id):
	"""Standard `Cancel' button for option windows"""

	window = GetWindow ()
	button = window.GetControl (button_id)
	button.SetText (STR_OPT_CANCEL) # Cancel
	button.OnPress (action)
	button.MakeEscape()

	if GameCheck.IsPST():
		button.SetVarAssoc ("Cancel", 1)

def OptHelpText (text_id, text_strref):
	"""Standard textarea with context help for option windows"""

	window = GetWindow ()
	text = window.GetControl (text_id)
	text.SetText (text_strref)
	text.AddAlias ("OPTHELP", 0, 1)
	text.SetValue (text_strref)
	return text

def OptBuddyLabel (nameID, strRef = None, focusedText = None, defaultText = None):
	window = GetWindow ()
	button = window.GetControl (nameID)
	button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
	button.SetState (IE_GUI_BUTTON_LOCKED)
	if strRef and GameCheck.IsPST ():
		button.SetText (strRef)
	help_ta = GemRB.GetView ("OPTHELP")
	if help_ta and HasMouseOver ():
		button.OnMouseEnter (lambda: help_ta.SetText (focusedText))
		button.OnMouseLeave (lambda: help_ta.SetText (defaultText))
	else:
		button.OnPress (lambda: help_ta.SetText (focusedText))

def GetWindow ():
	win = GemRB.GetView ("SUB_WIN", 1)
	return win if win else GemRB.GetView ("SUB_WIN", 0)

def HasMouseOver ():
	return GameCheck.IsPST () or GameCheck.IsIWD2 ()

###################################################
# End of file GUIOPTControls.py



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# 
import GemRB
import LoadScreen
from GUIDefines import *
from datetime import datetime

QuickLoadSlot = 0
StartWindow = None;

def OnLoad():
	global StartWindow, QuickLoadSlot

	skip_videos = GemRB.GetVar ("SkipIntroVideos")
	if not skip_videos:
		GemRB.PlayMovie ('BISLOGO', 1)
		GemRB.PlayMovie ('WOTC', 1)
		GemRB.PlayMovie ('NVIDIA', 1)
		GemRB.PlayMovie ('INTRO', 1)
		GemRB.SetVar ("SkipIntroVideos", 1)

#main window
	StartWindow = GemRB.LoadWindow(0, "GUICONN")
	StartWindow.AddAlias ("STARTWIN", 0, True)
	time = datetime.now()

	# IWD2 has some nice background for the night
	if time.hour >= 18 or time.hour <= 6:
		StartWindow.SetBackground ("STARTN");

		AnimButton = StartWindow.GetControl (0xfff0001)
		if not AnimButton:
			AnimButton = StartWindow.CreateButton (0xfff0001, 57, 333, 100, 100);
		AnimButton.SetAnimation ("MMTRCHB")
		AnimButton.SetState (IE_GUI_BUTTON_LOCKED)
		AnimButton.SetFlags (IE_GUI_BUTTON_CENTER_PICTURES, OP_OR)

	ProtocolButton = StartWindow.GetControl(0x00)
	NewGameButton = StartWindow.GetControl(0x02)
	LoadGameButton = StartWindow.GetControl(0x07)
	QuickLoadButton = StartWindow.GetControl(0x03)
	JoinGameButton = StartWindow.GetControl(0x0B)
	OptionsButton = StartWindow.GetControl(0x08)
	QuitGameButton = StartWindow.GetControl(0x01)
	VersionLabel = StartWindow.GetControl (0xfff0000)
	if not VersionLabel:
		VersionLabel = StartWindow.CreateLabel (0x0fff0000, 0,0,800,30, "REALMS2", "", IE_FONT_SINGLE_LINE | IE_FONT_ALIGN_CENTER)
	VersionLabel.SetText(GemRB.Version)
	ProtocolButton.SetStatus(IE_GUI_BUTTON_ENABLED)
	NewGameButton.SetStatus(IE_GUI_BUTTON_ENABLED)
	LoadGameButton.SetStatus(IE_GUI_BUTTON_ENABLED)

	GemRB.SetToken ("SaveDir", "mpsave")
	Games=GemRB.GetSaveGames()

	#looking for the quicksave
	EnableQuickLoad = IE_GUI_BUTTON_DISABLED
	for Game in Games:
		Slotname = Game.GetSaveID()
		# quick save is 1
		if Slotname == 1:
			EnableQuickLoad = IE_GUI_BUTTON_ENABLED
			QuickLoadSlot = Game
			break

	QuickLoadButton.SetStatus(EnableQuickLoad)
	JoinGameButton.SetStatus(IE_GUI_BUTTON_DISABLED)
	OptionsButton.SetStatus(IE_GUI_BUTTON_ENABLED)
	QuitGameButton.SetStatus(IE_GUI_BUTTON_ENABLED)
	LastProtocol = GemRB.GetVar("Last Protocol Used")
	if LastProtocol == 0:
		ProtocolButton.SetText(15413)
	elif LastProtocol == 1:
		ProtocolButton.SetText(13967)
	elif LastProtocol == 2:
		ProtocolButton.SetText(13968)
	NewGameButton.SetText(13963)
	LoadGameButton.SetText(13729)
	QuickLoadButton.SetText(33508)
	JoinGameButton.SetText(13964)
	OptionsButton.SetText(13905)
	QuitGameButton.SetText(13731)
	QuitGameButton.MakeEscape()
	NewGameButton.OnPress (NewGamePress)
	QuitGameButton.OnPress (QuitPress)
	ProtocolButton.OnPress (ProtocolPress)
	OptionsButton.OnPress (OptionsPress)
	LoadGameButton.OnPress (LoadPress)
	QuickLoadButton.OnPress (QuickLoadPress)
	StartWindow.Focus()
	GemRB.LoadMusicPL("Theme.mus")
	
	StartWindow.SetAction(RefreshProtocol, ACTION_WINDOW_FOCUS_GAINED)

	return

def ProtocolPress():
	ProtocolWindow = GemRB.LoadWindow(1, "GUICONN")

	#Disabling Unused Buttons in this Window
	Button = ProtocolWindow.GetControl(2)
	Button.SetState(IE_GUI_BUTTON_DISABLED)
	Button.SetFlags(IE_GUI_BUTTON_NO_IMAGE, OP_OR)
	Button = ProtocolWindow.GetControl(3)
	Button.SetState(IE_GUI_BUTTON_DISABLED)
	Button.SetFlags(IE_GUI_BUTTON_NO_IMAGE, OP_OR)
	Button = ProtocolWindow.GetControl(9)
	Button.SetState(IE_GUI_BUTTON_DISABLED)
	Button.SetFlags(IE_GUI_BUTTON_NO_IMAGE, OP_OR)

	SinglePlayerButton = ProtocolWindow.GetControl(10)
	SinglePlayerButton.SetFlags(IE_GUI_BUTTON_RADIOBUTTON,OP_OR)
	SinglePlayerButton.SetText(15413)

	IPXButton = ProtocolWindow.GetControl(0)
	IPXButton.SetFlags(IE_GUI_BUTTON_RADIOBUTTON,OP_OR)
	IPXButton.SetText(13967)

	TCPIPButton = ProtocolWindow.GetControl(1)
	TCPIPButton.SetFlags(IE_GUI_BUTTON_RADIOBUTTON,OP_OR)
	TCPIPButton.SetText(13968)

	SinglePlayerButton.SetVarAssoc("Last Protocol Used", 0)
	IPXButton.SetVarAssoc("Last Protocol Used", 1)
	TCPIPButton.SetVarAssoc("Last Protocol Used", 2)

	TextArea = ProtocolWindow.GetControl(7)
	TextArea.SetText(11316)

	DoneButton = ProtocolWindow.GetControl(6)
	DoneButton.SetText(11973)
	DoneButton.OnPress (ProtocolWindow.Close)
	DoneButton.MakeEscape()

	ProtocolWindow.Focus()
	return

def RefreshProtocol(win):
	ProtocolButton = win.GetControl(0)

	LastProtocol = GemRB.GetVar("Last Protocol Used")
	if LastProtocol == 0:
		ProtocolButton.SetText(15413)
	elif LastProtocol == 1:
		ProtocolButton.SetText(13967)
	elif LastProtocol == 2:
		ProtocolButton.SetText(13968)

	return

def LoadPress():
	GemRB.SetNextScript("GUILOAD")
	return

def QuickLoadPress():
	global QuickLoadSlot

	LoadScreen.StartLoadScreen()
	GemRB.LoadGame(QuickLoadSlot) # load & start game
	GemRB.EnterGame()
	return

def OptionsPress():
	GemRB.SetNextScript("Options")
	return

def QuitPress():
	QuitWindow = GemRB.LoadWindow(22, "GUICONN")
	CancelButton = QuitWindow.GetControl(2)
	CancelButton.OnPress (QuitWindow.Close)
	CancelButton.MakeEscape()

	QuitButton = QuitWindow.GetControl(1)
	QuitButton.OnPress (lambda: GemRB.Quit())
	QuitButton.MakeDefault()

	TextArea = QuitWindow.GetControl(0)
	CancelButton.SetText(13727)
	QuitButton.SetText(15417)
	TextArea.SetText(19532)
	QuitWindow.ShowModal (MODAL_SHADOW_GRAY)
	return

def NewGamePress():
	GemRB.SetNextScript("SPParty")
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2014 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#
# GameCheck.py - functions to check GameType

import os

import GemRB
from ie_restype import RES_WMP, RES_ARE, RES_2DA
from GUIDefines import SV_GAMEPATH

MAX_PARTY_SIZE = GemRB.GetVar ("MaxPartySize")

def IsPST ():
	return GemRB.GameType == "pst"

def IsIWD ():
	return GemRB.GameType == "iwd"

def IsHOW ():
	return GemRB.GameType == "how"

def IsIWD1 ():
	return GemRB.GameType == "iwd" or GemRB.GameType == "how"

def IsIWD2 ():
	return GemRB.GameType == "iwd2"

def IsBG1 ():
	return GemRB.GameType == "bg1"

def IsBG2 ():
	return GemRB.GameType == "bg2"

def IsBG2OrEE ():
	return GemRB.GameType in ["bg2", "bg2ee"]

def IsBG2EE ():
	return GemRB.GameType == "bg2ee"

def IsBG2Demo ():
	return ('BG2Demo' in GemRB.__dict__) and (GemRB.BG2Demo == True)

def IsGemRBDemo ():
	return GemRB.GameType == "demo"

def IsTOB ():
	return GemRB.HasResource ("worldm25", RES_WMP) and GemRB.GetVar("oldgame") == 0

def HasTOB ():
	return GemRB.HasResource ("worldm25", RES_WMP)

def HasHOW ():
	return GemRB.HasResource ("expmap", RES_WMP)

def HasTOTL ():
	return GemRB.HasResource ("ar9700", RES_ARE)

def HasBGT ():
	return GemRB.HasResource ("ar7200", RES_ARE)

def HasTutu ():
	return GemRB.HasResource ("fw0125", RES_ARE)

def HasTOTSC ():
	return GemRB.HasResource ("toscst", RES_2DA)

# there are no marker files, so check weidu.log
def HasWideScreenMod ():
	gamePath = GemRB.GetSystemVariable (SV_GAMEPATH)
	weiduLogPath = os.path.join (gamePath, "weidu.log")
	try:
		# latin-1 accepts all 1B codes and we only check the ASCII subset;
		# the log file appears to be inconsistent in some cases
		weiduLog = open (weiduLogPath, encoding='latin-1')
	except OSError:
		return False

	ret = False
	for line in weiduLog:
		if "~WIDESCREEN/" in line and not "Uninstalled" in line:
			ret = True
			break

	weiduLog.close ()
	return ret



# GemRB - Infinity Engine Emulator
# Copyright (C) 2024 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# ActionsWindow.py - functions to set up the main action bar (see UpdateActionsWindow)

import GemRB
import GameCheck
import GUICommon
import CommonTables
if not GameCheck.IsPST ():
  import Spellbook
from Clock import UpdateClock, CreateClockButton
from GUIDefines import *
from ie_action import *
from ie_modal import *
from ie_slots import SLOT_QUIVER
from ie_sounds import CHAN_HITS
from ie_stats import *

CurrentWindow = None
ActionBarControlOffset = 0
if GameCheck.IsIWD2 ():
	ActionBarControlOffset = 6 # portrait and action window were merged
fistDrawn = True

# The following four functions are currently unused in pst
def EmptyControls ():
	if GameCheck.IsPST():
		return
	Selected = GemRB.GetSelectedSize()
	if Selected==1:
		pc = GemRB.GameGetFirstSelectedActor ()
		# init spell list
		GemRB.SpellCast (pc, -1, 0)

	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	if not CurrentWindow:
		return # current case in our game demo (on right-click)
	for i in range (12):
		Button = CurrentWindow.GetControl (i+ActionBarControlOffset)
		Button.SetVisible (False)
	return

def SelectFormation (btn):
	GemRB.GameSetFormation (btn.Value)
	return

def SelectFormationPreset ():
	"""Choose the default formation."""

	GemRB.GameSetFormation (GemRB.GetVar ("Value"), GemRB.GetVar ("Formation")) # save
	GemRB.GameSetFormation (GemRB.GetVar ("Formation")) # set
	GroupControls ()
	return

def SetupFormation ():
	"""Opens the formation selection section."""

	for i in range (12):
		Button = CurrentWindow.GetControl (i+ActionBarControlOffset)
		Button.SetFlags (IE_GUI_BUTTON_NORMAL, OP_SET)
		Button.SetSprites ("GUIBTBUT", 0, 0, 1, 2, 3)
		Button.SetBAM ("FORM%x"%i, 0, 0, -1)
		Button.SetVarAssoc ("Value", i)
		Button.OnPress (SelectFormationPreset)
		Button.SetState (IE_GUI_BUTTON_UNPRESSED)
	return

def GroupControls ():
	"""Sections that control group actions."""

	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	Button = CurrentWindow.GetControl (ActionBarControlOffset)
	if GameCheck.IsBG2OrEE ():
		Button.SetActionIcon (globals(), 7, 1) # talk icon
	else:
		Button.SetActionIcon (globals(), 14, 1)# guard icon
	Button = CurrentWindow.GetControl (1 + ActionBarControlOffset)
	Button.SetActionIcon (globals(), 15, 2)
	Button = CurrentWindow.GetControl (2 + ActionBarControlOffset)
	Button.SetActionIcon (globals(), 21, 3)
	Button = CurrentWindow.GetControl (3 + ActionBarControlOffset)
	Button.SetActionIcon (globals(), -1, 4)
	Button = CurrentWindow.GetControl (4 + ActionBarControlOffset)
	Button.SetActionIcon (globals(), -1, 5)
	Button = CurrentWindow.GetControl (5 + ActionBarControlOffset)
	Button.SetActionIcon (globals(), -1, 6)
	Button = CurrentWindow.GetControl (6+ActionBarControlOffset)
	Button.SetActionIcon (globals(), -1, 7)

	formation = GemRB.GameGetFormation () # formation index
	GemRB.SetVar ("Formation", formation)
	for i in range (5):
		Button = CurrentWindow.GetControl (7 + ActionBarControlOffset + i)
		idx = GemRB.GameGetFormation (i)
		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON | IE_GUI_BUTTON_NORMAL, OP_SET)
		# kill the previous sprites or they show through
		Button.SetSprites ("GUIBTBUT", 0, 0, 1, 2, 3)
		Button.SetBAM ("FORM%x" % idx, 0, 0, -1)
		Button.SetVarAssoc ("Formation", i)
		Button.OnPress (SelectFormation)
		Button.OnRightPress (SetupFormation)
		Button.SetTooltip (4935)
		# 0x90 = F1 key
		Button.SetHotKey (chr(7 + i + 0x90), 0, True)

	# work around radiobutton preselection issue
	for i in range (5):
		Button = CurrentWindow.GetControl (7 + ActionBarControlOffset + i)
		if i == formation:
			Button.SetState (IE_GUI_BUTTON_SELECTED)
		else:
			Button.SetState (IE_GUI_BUTTON_ENABLED)

	return

def OpenActionsWindowControls (Window):
	CreateClockButton(Window.GetControl (62))
	Window.AddAlias ("ACTWIN") # needs to happen before the first update or it will be a noop
	UpdateActionsWindow ()
	return

## not used in pst - not sure any items have abilities, but it is worth making a note to find out
def SelectItemAbility():
	pc = GemRB.GameGetFirstSelectedActor ()
	slot = GemRB.GetVar ("Slot")
	ability = GemRB.GetVar ("Ability")
	GemRB.SetupQuickSlot (pc, 0, slot, ability)
	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	return

# in pst only nordom has bolts and they show on the same floatmenu as quickweapons, so needs work here
def SelectQuiverSlot():
	pc = GemRB.GameGetFirstSelectedActor ()
	slot = GemRB.GetVar ("Slot")
	slot_item = GemRB.GetSlotItem (pc, slot)
	# HACK: implement SetEquippedAmmunition instead?
	if not GemRB.IsDraggingItem ():
		item = GemRB.GetItem (slot_item["ItemResRef"])
		GemRB.DragItem (pc, slot, item["ItemIcon"]) #, 0, 0)
		GemRB.DropDraggedItem (pc, slot)
	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	return

# this doubles up as an ammo selector (not yet used in pst)
def SetupItemAbilities(pc, slot, only):
	slot_item = GemRB.GetSlotItem(pc, slot)
	if not slot_item:
		# CHIV: Could configure empty quickslots from the game screen ala spells heres
		return

	item = GemRB.GetItem (slot_item["ItemResRef"])
	Tips = item["Tooltips"]
	Locations = item["Locations"]

	# clear buttons here
	EmptyControls()

	# check A: whether ranged weapon and B: whether to bother at all
	ammotype = 0
	if item["Type"] == CommonTables.ItemType.GetRowIndex ("BOW"):
		ammotype = CommonTables.ItemType.GetRowIndex ("ARROW")
	elif item["Type"] == CommonTables.ItemType.GetRowIndex ("XBOW"):
		ammotype = CommonTables.ItemType.GetRowIndex ("BOLT")
	elif item["Type"] == CommonTables.ItemType.GetRowIndex ("SLING"):
		ammotype = CommonTables.ItemType.GetRowIndex ("BULLET")

	ammoSlotCount = 0
	if ammotype:
		ammoslots = GemRB.GetSlots(pc, SLOT_QUIVER, 1)
		currentammo = GemRB.GetEquippedAmmunition (pc)
		currentbutton = None
		for i in range (12):
			Button = CurrentWindow.GetControl (i + ActionBarControlOffset)
			if i < len(ammoslots):
				ammoslot = GemRB.GetSlotItem (pc, ammoslots[i])
				st = GemRB.GetSlotType (ammoslots[i])
				ammoitem = GemRB.GetItem (ammoslot['ItemResRef']) # needed to show the ammo count
				Tips = ammoitem["Tooltips"]
				# if this item is valid ammo and was really found in a quiver slot
				if ammoitem['Type'] == ammotype and st["Type"] == SLOT_QUIVER:
					ammoSlotCount += 1
					Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON | IE_GUI_BUTTON_ALIGN_BOTTOM | IE_GUI_BUTTON_ALIGN_RIGHT, OP_SET)
					Button.SetSprites ("GUIBTBUT", 0, 0, 1, 3, 5)
					Button.SetItemIcon (ammoslot['ItemResRef'])
					Button.SetText (str(ammoslot["Usages0"]))
					Button.OnPress (SelectQuiverSlot)
					Button.OnRightPress (SelectQuiverSlot)
					Button.SetVarAssoc ("Slot", ammoslots[i])
					if Tips[0] != -1:
						Button.SetTooltip (Tips[0])
					if currentammo == ammoslots[i]:
						currentbutton = Button

		if currentbutton:
			currentbutton.SetState (IE_GUI_BUTTON_SELECTED)

	# skip when there is only one choice
	if ammoSlotCount == 1:
		ammoSlotCount = 0

	# reset back to the main action bar if there are no extra headers or quivers
	reset = not ammoSlotCount

	# check for item abilities and skip irrelevant headers
	# for quick weapons only show weapon headers
	# for quick items only show the opposite
	# for scrolls only show the first (second header is for learning)
	# So depending on the context Staff of Magi will have none or 2
	if item["Type"] == CommonTables.ItemType.GetRowIndex ("SCROLL"):
		Tips = ()

	# skip launchers - handled above
	# gesen bow (bg2:bow19) has just a projectile header (not bow) for its special attack
	# TODO: we should append it to the list of ammo as a usability improvement
	if only == UAW_QWEAPONS and ammoSlotCount > 1:
		Tips = ()

	if len(Tips) > 0:
		reset = False
		rmax = min(len(Tips), 12-ammoSlotCount)

		# for mixed items, only show headers if there is more than one appropriate one
		weaps = sum([i == ITEM_LOC_WEAPON for i in Locations])
		if only == UAW_QWEAPONS and weaps == 1 and ammoSlotCount <= 1:
			rmax = 0
			reset = True
		abils = sum([i == ITEM_LOC_EQUIPMENT for i in Locations])
		if only == UAW_QITEMS and abils == 1:
			rmax = 0
			reset = True

		for i in range (rmax):
			if only == UAW_QITEMS:
				if Locations[i] != ITEM_LOC_EQUIPMENT:
					continue
			elif only == UAW_QWEAPONS:
				if Locations[i] != ITEM_LOC_WEAPON:
					continue
			Button = CurrentWindow.GetControl (i + ActionBarControlOffset + ammoSlotCount)
			Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON | IE_GUI_BUTTON_NORMAL, OP_SET)
			Button.SetSprites ("GUIBTBUT", 0, 0, 1, 2, 5)
			Button.SetItemIcon (slot_item['ItemResRef'], i + 6)
			Button.SetText ("")
			Button.OnPress (SelectItemAbility)
			Button.OnRightPress (SelectItemAbility)
			Button.SetVarAssoc ("Ability", i)
			Button.SetState (IE_GUI_BUTTON_ENABLED)
			if Tips[i] != -1:
				Button.SetTooltip ("F%d - %s" %(i, GemRB.GetString (Tips[i])))

	if reset:
		GemRB.SetVar ("ActionLevel", UAW_STANDARD)
		UpdateActionsWindow ()
	return

# iwd2 spell book/class selection
def SetupBookSelection ():
	pc = GemRB.GameGetFirstSelectedActor ()

	# get all the books that still have non-depleted memorisations
	# we need this list, so we can avoid holes in the action bar
	usableBooks = []
	for i in range (IE_IWD2_SPELL_SONG):
		bookClass = i
		if i == IE_IWD2_SPELL_INNATE: # shape stat comes later (8 vs 10)
			bookClass = IE_IWD2_SPELL_SHAPE

		enabled = False
		if i <= (IE_IWD2_SPELL_DOMAIN + 1): # booktypes up to and including domain + shapes
			spellCount = len(Spellbook.GetUsableMemorizedSpells (pc, bookClass))
			enabled = spellCount > 0
		if enabled:
			usableBooks.append(i)

	# if we have only one or only cleric+domain, skip to the spells
	bookCount = len(usableBooks)
	if bookCount == 1 or (bookCount == 2 and IE_IWD2_SPELL_CLERIC in usableBooks):
		GemRB.SetVar ("ActionLevel", UAW_SPELLS_DIRECT)
		UpdateActionsWindow ()
		return

	for i in range (12):
		Button = CurrentWindow.GetControl (i + ActionBarControlOffset)
		if i >= len(usableBooks):
			Button.SetActionIcon (globals(), -1)
			continue
		Button.SetActionIcon (globals(), 40 + usableBooks[i])
	return

# you can change this for custom skills, this is the original engine
skillbar = (ACT_STEALTH, ACT_SEARCH, ACT_THIEVING, ACT_WILDERNESS, ACT_TAMING, ACT_USE, ACT_CAST, 100, 100, 100, 100, 100)
def SetupSkillSelection ():
	pc = GemRB.GameGetFirstSelectedActor ()
	SetupControls (CurrentWindow, pc, ActionBarControlOffset, skillbar)
	return

def UpdateActionsWindow ():
	"""Redraws the actions section of the window."""

	global CurrentWindow
	global level, TopIndex

	# don't do anything if the GameControl is temporarily unavailable
	try:
		GUICommon.GameControl.GetFrame()
	except RuntimeError:
		return

	PortraitWin = GemRB.GetView("PORTWIN")
	ActionsWindow = GemRB.GetView("ACTWIN")

	if not GameCheck.IsIWD2():
		CurrentWindow = ActionsWindow
		ActionBarControlOffset = 0
		UpdateClock ()
	else:
		CurrentWindow = PortraitWin
		ActionBarControlOffset = 6 # set it here too, since we get called before menu setup

	if GameCheck.IsPST():
		return

	if CurrentWindow == None:
		return

	Selected = GemRB.GetSelectedSize()

	# setting up the disabled button overlay (using the second border slot)
	for i in range (12):
		Button = CurrentWindow.GetControl (i + ActionBarControlOffset)
		if GameCheck.IsBG1():
			color = {'r' : 0, 'g' : 254, 'b' :0, 'a' : 255}
			Button.SetBorder (0, color, 0, 0, Button.GetInsetFrame(6, 6, 4, 4))

		color = {'r' : 50, 'g' : 30, 'b' :10, 'a' : 120}
		Button.SetBorder (1, color, 0, 1)
		if GameCheck.IsBG2EE ():
			# for some reason NUMBER doesn't display
			Button.SetFont ("NUMBER2")
		else:
			Button.SetFont ("NUMBER")
		Button.SetText ("")
		Button.SetTooltip("")
		Button.SetDisabled (False)

	if Selected == 0:
		EmptyControls ()
		return
	if Selected > 1:
		GroupControls ()
		return

	# we are sure there is only one actor selected
	pc = GemRB.GameGetFirstSelectedActor ()

	# all but bg2 summons all use the group actionbar!
	# bg2 has a different common format and exceptions for illusions/clones
	level = GemRB.GetVar ("ActionLevel")
	if GemRB.GetPlayerStat (pc, IE_EA) >= CONTROLLED:
		if GameCheck.IsBG2OrEE ():
			if level == UAW_STANDARD and GemRB.GetPlayerStat (pc, IE_SEX) != 7:
				level = UAW_BG2SUMMONS
		else:
			GroupControls ()
			return

	TopIndex = GemRB.GetVar ("TopIndex")
	if level == UAW_STANDARD:
		# this is based on class
		# cleric/thief wants 13 icons, so we push the extra under innates (original did picking, we do turning)
		SetupControls (CurrentWindow, pc, ActionBarControlOffset)
	elif level == UAW_EQUIPMENT:
		CurrentWindow.SetupEquipmentIcons(globals(), pc, TopIndex, ActionBarControlOffset)
	elif level == UAW_SPELLS or level == UAW_SPELLS_DIRECT: # spells
		if GameCheck.IsIWD2():
			if level == UAW_SPELLS:
				# set up book selection if appropriate
				SetupBookSelection ()
				return
			# otherwise just throw everything in a single list
			# everything but innates, songs and shapes
			spelltype = (1 << IE_IWD2_SPELL_INNATE) - 1
		else:
			spelltype = (1 << IE_SPELL_TYPE_PRIEST) + (1 << IE_SPELL_TYPE_WIZARD)
		GemRB.SetVar ("Type", spelltype)
		Spellbook.SetupSpellIcons(CurrentWindow, spelltype, TopIndex, ActionBarControlOffset)
	elif level == UAW_INNATES: # innates
		if GameCheck.IsIWD2():
			spelltype = (1 << IE_IWD2_SPELL_INNATE) + (1 << IE_IWD2_SPELL_SHAPE)
		else:
			spelltype = 1 << IE_SPELL_TYPE_INNATE
		GemRB.SetVar ("Type", spelltype)
		Spellbook.SetupSpellIcons(CurrentWindow, spelltype, TopIndex, ActionBarControlOffset)
	elif level == UAW_QWEAPONS or level == UAW_QITEMS: # quick weapon or quick item ability selection
		SetupItemAbilities(pc, GemRB.GetVar("Slot"), level)
	elif level == UAW_BG2SUMMONS:
			SetupControls (CurrentWindow, pc, ActionBarControlOffset)
			# use group attack icon and disable second weapon slot
			Button = CurrentWindow.GetControl (1 + ActionBarControlOffset)
			Button.SetActionIcon (globals(), 15, 2)
			Button = CurrentWindow.GetControl (2 + ActionBarControlOffset)
			Button.SetActionIcon (globals(), -1, 0)
			# the games showed quick spells, but these actors don't have the data for it
			# we just disable the slots instead
			Button = CurrentWindow.GetControl (3 + ActionBarControlOffset)
			Button.SetActionIcon (globals(), 3, 4)
			Button.SetDisabled (True)
			Button = CurrentWindow.GetControl (4 + ActionBarControlOffset)
			Button.SetActionIcon (globals(), 4, 5)
			Button.SetDisabled (True)
			Button = CurrentWindow.GetControl (5 + ActionBarControlOffset)
			Button.SetActionIcon (globals(), 5, 6)
			Button.SetDisabled (True)
	elif level == UAW_ALLMAGE: # all known mage spells
		GemRB.SetVar ("Type", -1)
		Spellbook.SetupSpellIcons(CurrentWindow, -1, TopIndex, ActionBarControlOffset)
	elif level == UAW_SKILLS: # iwd2 skills
		SetupSkillSelection()
	elif level == UAW_QSPELLS: # quickspells, but with innates too
		if GameCheck.IsIWD2():
			spelltype = (1 << IE_IWD2_SPELL_INNATE) - 1
			spelltype += (1 << IE_IWD2_SPELL_INNATE) + (1 << IE_IWD2_SPELL_SHAPE)
		else:
			spelltype = (1 << IE_SPELL_TYPE_PRIEST) + (1 << IE_SPELL_TYPE_WIZARD) + (1 << IE_SPELL_TYPE_INNATE)
		GemRB.SetVar ("Type", spelltype)
		Spellbook.SetupSpellIcons(CurrentWindow, spelltype, TopIndex, ActionBarControlOffset)
	elif level == UAW_QSHAPES: # shapes selection
		spelltype = 1 << IE_IWD2_SPELL_SHAPE
		GemRB.SetVar ("Type", spelltype)
		Spellbook.SetupSpellIcons(CurrentWindow, spelltype, TopIndex, ActionBarControlOffset)
	elif level == UAW_QSONGS: # songs selection
		spelltype = 1 << IE_IWD2_SPELL_SONG
		if GameCheck.IsIWD1():
			spelltype = 1 << IE_SPELL_TYPE_SONG
		GemRB.SetVar ("Type", spelltype)
		Spellbook.SetupSpellIcons(CurrentWindow, spelltype, TopIndex, ActionBarControlOffset)
	elif level == UAW_BOOK: # spellbook selection
		spelltype = GemRB.GetVar ("Type")
		Spellbook.SetupSpellIcons(CurrentWindow, spelltype, TopIndex, ActionBarControlOffset)
	elif level == UAW_2DASPELLS: # spells from a 2da (fx_select_spell)
		if GameCheck.IsIWD2():
			# everything but innates, songs and shapes
			spelltype = (1 << IE_IWD2_SPELL_INNATE) - 1
		else:
			spelltype = (1 << IE_SPELL_TYPE_PRIEST) + (1 << IE_SPELL_TYPE_WIZARD)
		GemRB.SetVar ("Type", spelltype)
		Spellbook.SetupSpellIcons (CurrentWindow, spelltype, TopIndex, ActionBarControlOffset)
	else:
		print("Invalid action level:", level)
		GemRB.SetVar ("ActionLevel", UAW_STANDARD)
		UpdateActionsWindow ()
	return

def ActionQWeaponPressed (which):
	"""Selects the given quickslot weapon if possible."""

	pc = GemRB.GameGetFirstSelectedActor ()
	qs = GemRB.GetEquippedQuickSlot (pc, 1)

	# 38 is the magic slot
	if ((qs==which) or (qs==38)) and GemRB.GameControlGetTargetMode() != TARGET_MODE_ATTACK:
		GemRB.GameControlSetTargetMode (TARGET_MODE_ATTACK, GA_NO_DEAD | GA_NO_SELF | GA_NO_HIDDEN)
	else:
		GemRB.GameControlSetTargetMode (TARGET_MODE_NONE)
		GemRB.SetEquippedQuickSlot (pc, which, -1)

	SetupControls (CurrentWindow, pc, ActionBarControlOffset)
	UpdateActionsWindow ()
	return

# TODO: implement full weapon set switching instead
def ActionQWeaponRightPressed (action):
	"""Selects the used ability of the quick weapon."""
	GemRB.SetVar ("Slot", action)
	GemRB.SetVar ("ActionLevel", UAW_QWEAPONS)
	UpdateActionsWindow ()
	return

###############################################
# quick icons for spells, innates, songs, shapes

def ActionQSpellPressed (which):
	pc = GemRB.GameGetFirstSelectedActor ()

	GemRB.SpellCast (pc, -2, which)
	UpdateActionsWindow ()
	return

def ActionQSpellRightPressed (which):
	GemRB.SetVar ("QSpell", which)
	GemRB.SetVar ("TopIndex", 0)
	GemRB.SetVar ("ActionLevel", UAW_QSPELLS)
	UpdateActionsWindow ()
	return

suf = ["", "Right"]
# this function is used to generate various action bar actions
# that should be available in 9 slots (iwd2)
def GenerateButtonActions(num, name, g, right = 0, offset = 0):
	dec = "def Action" + name + str(num + 1) + suf[right] + "Pressed():\n"
	dec += "\tAction" + name + suf[right] + "Pressed(" + str(num + offset) + ")"
	exec(dec, g) # pass on the same global dict, so we remain in the top scope

for i in range(9):
	GenerateButtonActions(i, "QSpec", globals(), 0)
	GenerateButtonActions(i, "QSpec", globals(), 1)
	GenerateButtonActions(i, "QSpell", globals(), 0)
	GenerateButtonActions(i, "QSpell", globals(), 1)
	GenerateButtonActions(i, "QShape", globals(), 0)
	GenerateButtonActions(i, "QShape", globals(), 1)
	GenerateButtonActions(i, "QSong", globals(), 0)
	GenerateButtonActions(i, "QSong", globals(), 1)
for i in range(4):
	GenerateButtonActions(i, "QWeapon", globals(), 0)
	GenerateButtonActions(i, "QWeapon", globals(), 1, 10)

def ActionQSpecPressed (which):
	ActionQSpellPressed (which)

def ActionQSpecRightPressed (which):
	GemRB.SetVar ("QSpell", which)
	GemRB.SetVar ("TopIndex", 0)
	GemRB.SetVar ("ActionLevel", UAW_INNATES)
	UpdateActionsWindow ()

def ActionQShapePressed (which):
	ActionQSpellPressed (which)

def ActionQShapeRightPressed (which):
	GemRB.SetVar ("QSpell", which)
	GemRB.SetVar ("TopIndex", 0)
	GemRB.SetVar ("ActionLevel", UAW_QSHAPES)
	UpdateActionsWindow ()

def ActionQSongPressed (which):
	SelectBardSong (which) # TODO: verify parameter once we have actionbar customisation
	ActionBardSongPressed ()

def ActionQSongRightPressed (which):
	GemRB.SetVar ("QSpell", which)
	GemRB.SetVar ("TopIndex", 0)
	GemRB.SetVar ("ActionLevel", UAW_QSONGS)
	UpdateActionsWindow ()

# can't pass the globals dictionary from another module
def SetActionIconWorkaround(Button, action, function):
	Button.SetActionIcon (globals(), action, function)

# no check needed because the button wouldn't be drawn if illegal
def ActionLeftPressed ():
	"""Scrolls the actions window left.

	Used primarily for spell selection."""

	TopIndex = GemRB.GetVar ("TopIndex")
	if TopIndex>10:
		TopIndex -= 10
	else:
		TopIndex = 0
	GemRB.SetVar ("TopIndex", TopIndex)
	UpdateActionsWindow ()
	return

# no check needed because the button wouldn't be drawn if illegal
def ActionRightPressed ():
	"""Scrolls the action window right.

	Used primarily for spell selection."""

	pc = GemRB.GameGetFirstSelectedActor ()
	TopIndex = GemRB.GetVar ("TopIndex")
	Type = GemRB.GetVar ("Type")
	print("Type:", Type)
	# Type is a bitfield if there is no level given
	# This is to make sure cleric/mages get all spells listed
	if GemRB.GetVar ("ActionLevel") == UAW_ALLMAGE:
		if Type == 3:
			Max = len(Spellbook.GetKnownSpells (pc, IE_SPELL_TYPE_PRIEST) + Spellbook.GetKnownSpells (pc, IE_SPELL_TYPE_WIZARD))
		else:
			Max = GemRB.GetKnownSpellsCount (pc, Type, -1) # this can handle only one type at a time
	else:
		Max = GemRB.GetMemorizedSpellsCount(pc, Type, -1, 1)
	print("Max:", Max)
	TopIndex += 10
	if TopIndex > Max - 10:
		if Max>10:
			if TopIndex > Max:
				TopIndex = Max - 10
		else:
			TopIndex = 0
	GemRB.SetVar ("TopIndex", TopIndex)
	UpdateActionsWindow ()
	return

def ActionMeleePressed ():
	""" switches to the most damaging melee weapon"""

	# get the party Index
	pc = GemRB.GameGetFirstSelectedPC ()
	GemRB.ExecuteString("EquipMostDamagingMelee()", pc)
	return

def ActionRangePressed ():
	""" switches to the most damaging ranged weapon"""

	# get the party Index
	pc = GemRB.GameGetFirstSelectedPC ()
	GemRB.ExecuteString("EquipRanged()", pc)
	return

def ActionShapeChangePressed ():
	GemRB.SetVar ("ActionLevel", UAW_QSHAPES)
	GemRB.SetVar ("TopIndex", 0)
	UpdateActionsWindow ()
	return

def SelectBardSong (which):
	songType = IE_IWD2_SPELL_SONG
	if GameCheck.IsIWD1 ():
		songType = IE_SPELL_TYPE_SONG
	pc = GemRB.GameGetFirstSelectedActor ()
	songs = Spellbook.GetKnownSpells (pc, songType)
	# "which" is a mashup of the spell index with it's type
	idx = which % ((1 << songType) * 100)
	qsong = songs[idx]['SpellResRef']
	# the effect needs to be set each tick, so we use FX_DURATION_INSTANT_PERMANENT==1 timing mode
	# GemRB.SetModalState can also set the spell, but it wouldn't persist
	GemRB.ApplyEffect (pc, 'ChangeBardSong', 0, idx, qsong, "", "", "", 1)

def ActionBardSongRightPressed ():
	"""Selects a bardsong."""
	GemRB.SetVar ("ActionLevel", UAW_QSONGS)
	GemRB.SetVar ("TopIndex", 0)
	UpdateActionsWindow ()
	return

def ActionBardSongPressed ():
	"""Toggles the battle song."""

	# get the global ID
	pc = GemRB.GameGetFirstSelectedActor ()
	GemRB.SetModalState (pc, MS_BATTLESONG)
	GemRB.PlaySound ("act_01")
	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	UpdateActionsWindow ()
	return

def ActionSearchPressed ():
	"""Toggles detect traps."""

	# get the global ID
	pc = GemRB.GameGetFirstSelectedActor ()
	GemRB.SetModalState (pc, MS_DETECTTRAPS)
	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	UpdateActionsWindow ()
	return

def ActionStealthPressed ():
	"""Toggles stealth."""

	pc = GemRB.GameGetFirstSelectedActor ()
	GemRB.SetModalState (pc, MS_STEALTH)
	GemRB.PlaySound ("act_07", CHAN_HITS)
	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	UpdateActionsWindow ()
	return

def ActionTurnPressed ():
	"""Toggles turn undead."""

	pc = GemRB.GameGetFirstSelectedActor ()
	GemRB.SetModalState (pc, MS_TURNUNDEAD)
	GemRB.PlaySound ("act_06")
	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	UpdateActionsWindow ()
	return

def ActionTamingPressed ():
	pc = GemRB.GameGetFirstSelectedActor ()
	GemRB.SpellCast (pc, -3, 0, "spin108")
	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	UpdateActionsWindow ()
	return

def ActionWildernessPressed ():
	pc = GemRB.GameGetFirstSelectedActor ()
	GemRB.ApplyEffect (pc, "Reveal:Tracks", 0, 0)
	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	UpdateActionsWindow ()
	return

def ActionDancePressed ():
	pc = GemRB.GameGetFirstSelectedActor ()
	GemRB.SetModalState (pc, MS_DANCE)
	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	UpdateActionsWindow ()
	return

def ActionUseItemPressed ():
	GemRB.SetVar ("TopIndex", 0)
	GemRB.SetVar ("ActionLevel", UAW_EQUIPMENT)
	UpdateActionsWindow ()
	return

def ActionCastPressed ():
	"""Opens the spell choice scrollbar."""

	GemRB.SetVar ("QSpell", -1)
	GemRB.SetVar ("TopIndex", 0)
	GemRB.SetVar ("ActionLevel", UAW_SPELLS)
	UpdateActionsWindow ()
	return

def ActionQItemPressed (action):
	"""Uses the given quick item."""

	pc = GemRB.GameGetFirstSelectedActor ()
	# quick slot
	GemRB.UseItem (pc, -2, action, -1)
	return

def ActionQItem1Pressed ():
	ActionQItemPressed (ACT_QSLOT1)
	return

def ActionQItem2Pressed ():
	ActionQItemPressed (ACT_QSLOT2)
	return

def ActionQItem3Pressed ():
	ActionQItemPressed (ACT_QSLOT3)
	return

def ActionQItem4Pressed ():
	ActionQItemPressed (ACT_QSLOT4)
	return

def ActionQItem5Pressed ():
	ActionQItemPressed (ACT_QSLOT5)
	return

def ActionQItemRightPressed (action):
	"""Selects the used ability of the quick item."""

	GemRB.SetVar ("Slot", action)
	GemRB.SetVar ("ActionLevel", UAW_QITEMS)
	UpdateActionsWindow ()
	return

def ActionQItem1RightPressed ():
	ActionQItemRightPressed (19)

def ActionQItem2RightPressed ():
	ActionQItemRightPressed (20)

def ActionQItem3RightPressed ():
	ActionQItemRightPressed (21)

def ActionQItem4RightPressed ():
	ActionQItemRightPressed (22)

def ActionQItem5RightPressed ():
	ActionQItemRightPressed (23)

def ActionInnatePressed ():
	"""Opens the innate spell scrollbar."""

	GemRB.SetVar ("QSpell", -1)
	GemRB.SetVar ("TopIndex", 0)
	GemRB.SetVar ("ActionLevel", UAW_INNATES)
	UpdateActionsWindow ()
	return

def ActionSkillsPressed ():
	GemRB.SetVar ("TopIndex", 0)
	GemRB.SetVar ("ActionLevel", UAW_SKILLS)
	UpdateActionsWindow ()
	return

def TypeSpellPressed (spelltype):
	GemRB.SetVar ("Type", 1 << spelltype)
	GemRB.SetVar ("ActionLevel", UAW_BOOK)
	UpdateActionsWindow ()
	return

def ActionBardSpellPressed ():
	TypeSpellPressed(IE_IWD2_SPELL_BARD)
	return

def ActionClericSpellPressed ():
	TypeSpellPressed(IE_IWD2_SPELL_CLERIC)
	return

def ActionDruidSpellPressed ():
	TypeSpellPressed(IE_IWD2_SPELL_DRUID)
	return

def ActionPaladinSpellPressed ():
	TypeSpellPressed(IE_IWD2_SPELL_PALADIN)
	return

def ActionRangerSpellPressed ():
	TypeSpellPressed(IE_IWD2_SPELL_RANGER)
	return

def ActionSorcererSpellPressed ():
	TypeSpellPressed(IE_IWD2_SPELL_SORCERER)
	return

def ActionWizardSpellPressed ():
	TypeSpellPressed(IE_IWD2_SPELL_WIZARD)
	return

def ActionDomainSpellPressed ():
	TypeSpellPressed(IE_IWD2_SPELL_DOMAIN)
	return

def ActionWildShapesPressed ():
	TypeSpellPressed(IE_IWD2_SPELL_SHAPE)
	return

def SpellShiftPressed ():
	Spell = GemRB.GetVar ("Spell") # spellindex from spellbook jumbled with booktype
	Type =  Spell // 1000
	SpellIndex = Spell % 1000

	# try spontaneous casting
	pc = GemRB.GameGetFirstSelectedActor ()
	ClassRowName = GUICommon.GetClassRowName (pc)
	SponCastTableName = CommonTables.ClassSkills.GetValue (ClassRowName, "SPONCAST")
	if SponCastTableName == "*":
		# proceed as if nothing happened
		SpellPressed ()
		return

	SponCastTable = GemRB.LoadTable (SponCastTableName, True, True)
	if not SponCastTable:
		print("SpellShiftPressed: skipping, non-existent spontaneous casting table used! ResRef:", SponCastTableName)
		SpellPressed ()
		return

	# determine the column number (spell variety) depending on alignment
	CureOrHarm = GemRB.GetPlayerStat (pc, IE_ALIGNMENT)
	if CureOrHarm % 16 == 3: # evil
		CureOrHarm = 1
	else:
		CureOrHarm = 0

	# get the unshifted booktype
	BaseType = 0
	tmp = Type
	while tmp > 1:
		tmp = tmp >> 1
		BaseType += 1

	# figure out the spell's details
	# TODO: find a simpler way
	Spell = None
	MemorisedSpells = Spellbook.GetSpellinfoSpells (pc, BaseType)
	for spell in MemorisedSpells:
		if spell['SpellIndex'] % (255000) == SpellIndex: # 255 is the engine value of Type
			Spell = spell
			break

	# rownames==level; col1: good+neutral; col2: evil resref
	Level = Spell['SpellLevel']
	ReplacementSpell = SponCastTable.GetValue (Level - 1, CureOrHarm).upper()
	if ReplacementSpell != Spell['SpellResRef'].upper():
		SpellIndex = GemRB.PrepareSpontaneousCast (pc, Spell['SpellResRef'], Spell['BookType'], Level, ReplacementSpell)
		GemRB.SetVar ("Spell", SpellIndex + 1000 * Type)
		if GameCheck.IsIWD2 ():
			GemRB.DisplayString (39742, ColorWhite, pc) # Spontaneous Casting

	SpellPressed ()

# This is the endpoint for spellcasting, finally calling SpellCast. This always happens at least
# twice though, the second time to reset the action bar (more if wild magic or subspell selection is involved).
# Spell and Type (spellbook type) are set during the spell bar construction/use, which is in turn
# affected by ActionLevel (see UpdateActionsWindow of this module).
# Keep in mind, that the core resets Type and/or ActionLevel in the case of subspells (fx_select_spell).
def SpellPressed ():
	"""Prepares a spell to be cast."""

	pc = GemRB.GameGetFirstSelectedActor ()

	Spell = GemRB.GetVar ("Spell")
	Type = GemRB.GetVar ("Type")

	if Type == 1 << IE_IWD2_SPELL_SONG or (Type == 1 << IE_SPELL_TYPE_SONG and not GameCheck.IsIWD2()):
		SelectBardSong (Spell)
		ActionBardSongPressed()
		return

	GemRB.GameControlSetTargetMode (TARGET_MODE_CAST)
	if Type != -1:
		Type = Spell // 1000
	Spell = Spell % 1000
	slot = GemRB.GetVar ("QSpell")
	if slot is not None:
		# setup quickspell slot
		# if spell has no target, return
		# otherwise continue with casting
		Target = GemRB.SetupQuickSpell (pc, slot, Spell, Type)
		# sabotage the immediate casting of self targeting spells
		if Target == 5 or Target == 7:
			Type = -1
			GemRB.GameControlSetTargetMode (TARGET_MODE_NONE)

	if Type==-1:
		GemRB.SetVar ("ActionLevel", UAW_STANDARD)
		GemRB.SetVar("Type", 0)
	GemRB.SpellCast (pc, Type, Spell)
	if GemRB.GetVar ("Type") is not None:
		GemRB.SetVar ("ActionLevel", UAW_STANDARD)
		# init spell list
		GemRB.SpellCast (pc, -1, 0)
	GemRB.SetVar ("TopIndex", 0)
	UpdateActionsWindow ()
	return

def EquipmentPressed ():
	pc = GemRB.GameGetFirstSelectedActor ()

	GemRB.GameControlSetTargetMode (TARGET_MODE_CAST)
	Item = GemRB.GetVar ("Equipment")
	# equipment index
	GemRB.UseItem (pc, -1, Item, -1)
	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
	UpdateActionsWindow ()
	return

def ActionStopPressed ():
	for i in GemRB.GetSelectedActors():
		GemRB.ClearActions (i)
	return

def ActionTalkPressed ():
	GemRB.GameControlSetTargetMode (TARGET_MODE_TALK, GA_NO_DEAD | GA_NO_ENEMY | GA_NO_HIDDEN)

def ActionAttackPressed ():
	GemRB.GameControlSetTargetMode (TARGET_MODE_ATTACK, GA_NO_DEAD | GA_NO_SELF | GA_NO_HIDDEN)

def ActionDefendPressed ():
	GemRB.GameControlSetTargetMode (TARGET_MODE_DEFEND, GA_NO_SELF | GA_NO_ENEMY | GA_NO_HIDDEN)

def ActionThievingPressed ():
	GemRB.GameControlSetTargetMode (TARGET_MODE_PICK, GA_NO_DEAD | GA_NO_SELF | GA_NO_ENEMY | GA_NO_HIDDEN)

def SetupControls (Window, pc, actionOffset, customBar = None):
	global fistDrawn

	if customBar:
		actionRow = customBar
	else:
		actionRow = GemRB.GetPlayerActionRow (pc)

	pcStats = GemRB.GetPCStats (pc) # will be None for summons, familiars!
	invInfo = GemRB.GetInventoryInfo (pc)
	fistDrawn = True

	GUIBT_COUNT = 12
	for i in range(GUIBT_COUNT):
		btn = Window.GetControl (i + actionOffset)
		if not btn:
			raise RuntimeError("Missing action buttons!")

		action = actionRow[i]
		if action == ACT_NONE:
			action = -1

		btn.SetFlags (IE_GUI_BUTTON_NO_IMAGE | IE_GUI_BUTTON_ALIGN_BOTTOM | IE_GUI_BUTTON_ALIGN_RIGHT, OP_SET)
		ret = btn.SetActionIcon (globals(), action, i + 1)
		if ret is not None:
			raise RuntimeError("Cannot set action button {} to {}!".format(i, action))

		if action != -1:
			# reset it to the first one, so we can handle them more easily below
			if (action >= ACT_IWDQSPELL) and (action <= ACT_IWDQSPELL + 9):
				action = ACT_IWDQSPELL
			elif (action >= ACT_IWDQITEM) and (action <= ACT_IWDQITEM + 9):
				action = ACT_IWDQITEM
			elif (action >= ACT_IWDQSPEC) and (action <= ACT_IWDQSPEC + 9):
				action = ACT_IWDQSPEC
			elif (action >= ACT_IWDQSONG) and (action <= ACT_IWDQSONG + 9):
				action = ACT_IWDQSONG

		state = SetupActionButton (pc, action, btn, i, pcStats, invInfo)

		disabledButton = GemRB.GetPlayerStat (pc, IE_DISABLEDBUTTON)
		disablableAction = (action <= ACT_SKILLS) or (action >= ACT_QSPELL1 and action <= ACT_QSPELL3)
		# two entries don't match between the effect param and the constants
		# param          our values
		# 10 bardsong    ACT_QSLOT4=10  ACT_BARDSONG=20
		# 14 find traps  ACT_DEFEND=14  ACT_SEARCH=22
		if disabledButton & (1 << 10) and action == 20:
			action = 10
		elif disabledButton & (1 << 14) and action == 22:
			action = 14
		if action < 0 or (disablableAction and disabledButton & (1 << action)):
			state = IE_GUI_BUTTON_DISABLED

		btn.SetState (state)
		# you have to set this overlay up
		# this state check looks bizarre, but without it most buttons get misrendered
		btn.EnableBorder (1, state == IE_GUI_BUTTON_DISABLED)

def SetupActionButton (pc, action, btn, i, pcStats, invInfo):
	global fistDrawn

	state = IE_GUI_BUTTON_UNPRESSED
	magicSlot = invInfo["MagicSlot"]
	fistSlot = invInfo["FistSlot"]
	weaponSlot = invInfo["WeaponSlot"]
	usedslot = invInfo["UsedSlot"]
	modalState = GemRB.GetModalState (pc)

	def SetItemText (btn, charges, oneIsNone):
		if not btn:
			return

		if charges and (charges > 1 or not oneIsNone):
			btn.SetText (str(charges))
		else:
			btn.SetText (None)

	def SetQSpellBtn (btn, pc, tmp):
		if not pcStats:
			return IE_GUI_BUTTON_DISABLED

		poi = pcStats["QuickSpells"][tmp]
		if poi == "":
			return IE_GUI_BUTTON_FAKEDISABLED

		btn.SetSpellIcon (poi, 1, 1, i + 1)
		memorized = Spellbook.HasSpellinfoSpell (pc, poi)
		if not memorized:
			return IE_GUI_BUTTON_FAKEDISABLED # so right-click can still be invoked to change the binding

		# sigh, get unshifted value back
		bookType = pcStats["QuickSpellsBookType"][tmp]
		if bookType > 0:
			bookType = bin(pcStats["QuickSpellsBookType"][tmp])[::-1].index("1")
		memorizedSpells = Spellbook.GetUsableMemorizedSpells (pc, bookType)
		memorizedCount = 0
		for sp in memorizedSpells:
			if sp["SpellResRef"] == poi:
				memorizedCount = sp["MemoCount"]
				break
		if memorizedCount:
			SetItemText (btn, memorizedCount, True)
			return IE_GUI_BUTTON_UNPRESSED
		else:
			return IE_GUI_BUTTON_FAKEDISABLED

	def SetQSlotBtn (btn, pc, tmp, action):
		btn.SetBAM ("stonitem", 0, 0)
		if not pcStats:
			return IE_GUI_BUTTON_DISABLED
		slot = pcStats["QuickItemSlots"][tmp]
		if slot != 0xffff:
			# no slot translation required
			item = GemRB.GetSlotItem (pc, slot, 1)
			if item:
				# MISC3H (horn of blasting) is not displayed when it is out of usages
				header = pcStats["QuickItemHeaders"][tmp]
				usages = item["Usages" + str(header)]
				# I don't like this feature, if the goal is full IE compatibility
				# make the next two lines conditional on usages != 0
				# SetItemIcon parameter needs header + 6 to display extended header icons
				btn.SetItemIcon (item["ItemResRef"], header + 6, 2 if item["Flags"] & IE_INV_ITEM_IDENTIFIED else 1, i + 1)
				SetItemText (btn, usages, False)
			elif action == ACT_IWDQITEM:
				return IE_GUI_BUTTON_DISABLED
		elif action == ACT_IWDQITEM:
			return IE_GUI_BUTTON_DISABLED
		return IE_GUI_BUTTON_UNPRESSED

	def CanUseActionButton (pc, btnType):
		capability = -1
		if GameCheck.IsIWD2 ():
			if btnType == ACT_STEALTH:
				# iwd2 will automatically use the GetSkill checks for trained skills
				capability = GemRB.GetPlayerStat (pc, IE_STEALTH) + GemRB.GetPlayerStat (pc, IE_HIDEINSHADOWS)
			elif btnType == ACT_THIEVING:
				capability = GemRB.GetPlayerStat (pc, IE_LOCKPICKING) + GemRB.GetPlayerStat (pc, IE_PICKPOCKET)
			elif btnType == ACT_SEARCH:
				capability = 1 # everyone can try to search
			else:
				print("Unknown action (button) type: ", btnType)
		else:
			# use levels instead, so inactive dualclasses work as expected
			if btnType == ACT_STEALTH:
				capability = GemRB.GetPlayerLevel (pc, ISTHIEF) + GemRB.GetPlayerLevel (pc, ISMONK) + GemRB.GetPlayerLevel (pc, ISRANGER)
			elif btnType == ACT_THIEVING:
				capability = GemRB.GetPlayerLevel (pc, ISTHIEF) + GemRB.GetPlayerLevel (pc, ISBARD)
			elif btnType == ACT_SEARCH:
				capability = GemRB.GetPlayerLevel (pc, ISTHIEF) + GemRB.GetPlayerLevel (pc, ISSHAMAN)
			else:
				print("Unknown action (button) type: ", btnType)
		return capability > 0

	def HasAnyActiveCasterLevel (pc):
		levels = GemRB.GetPlayerLevel (pc, ISMAGE) + GemRB.GetPlayerLevel (pc, ISSORCERER) + GemRB.GetPlayerLevel (pc, ISBARD)
		levels = levels + GemRB.GetPlayerLevel (pc, ISCLERIC) + GemRB.GetPlayerLevel (pc, ISDRUID) + GemRB.GetPlayerLevel (pc, ISSHAMAN)
		# first spells at level: 9 (bg1), 6 (iwd), 4 (iwd2)
		# lowest works here, since we also check for spell presence
		# — either you have spells from the class or the other one provides them
		levels = levels + int(GemRB.GetPlayerLevel (pc, ISPALADIN) >= 4)
		# first spells at level: 8 (bg1), 6 (iwd), 4 (iwd2)
		# we could look it up in the tables instead
		levels = levels + int(GemRB.GetPlayerLevel(pc, ISRANGER) >= 4)
		return levels > 0

	SetItemText (btn, 0, False)

	########################################################################
	# big main switch
	if action == ACT_INNATE:
		if GameCheck.IsIWD2 ():
			spellType = (1 << IE_IWD2_SPELL_INNATE) | (1 << IE_IWD2_SPELL_SHAPE)
		else:
			spellType = 1 << IE_SPELL_TYPE_INNATE

		if len(GemRB.GetSpelldata (pc, spellType)) == 0:
			state = IE_GUI_BUTTON_DISABLED
	elif action == ACT_CAST:
		# luckily the castable spells in IWD2 are all bits below INNATE, so we can do this trick
		if GameCheck.IsIWD2 ():
			spellType = (1 << IE_IWD2_SPELL_INNATE) - 1
		else:
			spellType = (1 << IE_SPELL_TYPE_INNATE) - 1

		# returns true if there are ANY spells to cast
		if len(GemRB.GetSpelldata (pc, spellType)) == 0 or not HasAnyActiveCasterLevel (pc):
			state = IE_GUI_BUTTON_DISABLED
	elif action in [ACT_BARD, ACT_CLERIC, ACT_DRUID, ACT_PALADIN, ACT_RANGER, ACT_SORCERER, ACT_WIZARD, ACT_DOMAIN]:
		if GameCheck.IsIWD2 ():
			spellType = 1 << (action - ACT_BARD)
		else:
			# only cleric or wizard switch exists in the bg engine
			if action == ACT_WIZARD:
				spellType = 1 << IE_SPELL_TYPE_WIZARD
			else:
				spellType = 1 << IE_SPELL_TYPE_PRIEST

		# returns true if there is ANY spell
		if len(GemRB.GetSpelldata (pc, spellType)) == 0:
			state = IE_GUI_BUTTON_DISABLED
	elif action == ACT_WILDSHAPE or action == ACT_SHAPE:
		if GameCheck.IsIWD2 ():
			spellType = 1 << IE_IWD2_SPELL_SHAPE
		else:
			spellType = 0 # no separate shapes in old spellbook

		# returns true if there is ANY shape
		if len(GemRB.GetSpelldata (pc, spellType)) == 0:
			state = IE_GUI_BUTTON_DISABLED
	elif action == ACT_USE:
		# returns true if there is ANY equipment with a usable header
		if not invInfo["HasEquippedAbilities"]:
			state = IE_GUI_BUTTON_DISABLED
	elif action == ACT_BARDSONG:
		if GameCheck.IsIWD2 ():
			spellType = IE_IWD2_SPELL_SONG
			if not GemRB.GetKnownSpellsCount (pc, spellType):
				state = IE_GUI_BUTTON_DISABLED
			elif modalState == MS_BATTLESONG:
				state = IE_GUI_BUTTON_SELECTED
		elif modalState == MS_BATTLESONG:
			state = IE_GUI_BUTTON_SELECTED
	elif action == ACT_TURN:
		if GemRB.GetPlayerStat (pc, IE_TURNUNDEADLEVEL) < 1:
			state = IE_GUI_BUTTON_DISABLED
		elif modalState == MS_TURNUNDEAD:
			state = IE_GUI_BUTTON_SELECTED
	elif action == ACT_STEALTH:
		if not CanUseActionButton(pc, action):
			state = IE_GUI_BUTTON_DISABLED
		elif modalState == MS_STEALTH:
			state = IE_GUI_BUTTON_SELECTED
	elif action == ACT_SEARCH:
		# in IWD2 everyone can try to search, in bg2 only thieves get the icon
		if not CanUseActionButton(pc, action):
			state = IE_GUI_BUTTON_DISABLED
		elif modalState == MS_DETECTTRAPS:
			state = IE_GUI_BUTTON_SELECTED
	elif action == ACT_THIEVING:
		if not CanUseActionButton(pc, action):
			state = IE_GUI_BUTTON_DISABLED
	elif action == ACT_DANCE:
		if modalState == MS_DANCE:
			state = IE_GUI_BUTTON_SELECTED
	elif action == ACT_TAMING:
		if GemRB.GetPlayerStat (pc, IE_ANIMALS) <= 0:
			state = IE_GUI_BUTTON_DISABLED
	elif action in [ACT_WEAPON1, ACT_WEAPON2, ACT_WEAPON3, ACT_WEAPON4]:
		btn.SetBAM ("stonweap", 0, 0)
		if magicSlot == None:
			if pcStats:
				slot = pcStats["QuickWeaponSlots"][action - ACT_WEAPON1]
			else:
				slot = weaponSlot + (action - ACT_WEAPON1)
		else:
			slot = magicSlot

		item2ResRef = ""
		skip = False
		if slot == 0xffff:
			skip = True
		if not skip:
			item = GemRB.GetSlotItem (pc, slot, 1)
			if item:
				# no slot translation required
				launcherSlot = item["LauncherSlot"]
				if launcherSlot and launcherSlot != fistSlot:
					# launcher/projectile in this slot
					item2 = GemRB.GetSlotItem (pc, launcherSlot, 1)
					item2ResRef = item2["ItemResRef"]
			else:
				skip = True
		if not skip:
			if slot == fistSlot:
				if fistDrawn:
					fistDrawn = False
				else:
					# empty weapon slot, already drawn
					skip = True

		if not skip:
			btn.SetItemIcon (item["ItemResRef"], 4, 2 if item["Flags"] & IE_INV_ITEM_IDENTIFIED else 1, i + 1, item2ResRef)
			if pcStats:
				SetItemText (btn, item["Usages" + str(pcStats["QuickWeaponHeaders"][action - ACT_WEAPON1])], True)
			else:
				SetItemText (btn, 0, True)
			if usedslot == slot:
				btn.EnableBorder (0, True)
				if GemRB.GameControlGetTargetMode () == TARGET_MODE_ATTACK:
					state = IE_GUI_BUTTON_SELECTED
				else:
					state = IE_GUI_BUTTON_FAKEDISABLED
			else:
				btn.EnableBorder (0, False)
	elif action == ACT_IWDQSPELL:
		btn.SetBAM ("stonspel", 0, 0)
		if GameCheck.IsIWD2 () and i > 3:
			tmp = i - 3
		else:
			tmp = 0
		state = SetQSpellBtn (btn, pc, tmp)
	elif action == ACT_IWDQSONG:
		btn.SetBAM ("stonsong", 0, 0)
		if GameCheck.IsIWD2 () and i > 3:
			tmp = i - 3
		else:
			tmp = 0
		state = SetQSpellBtn (btn, pc, tmp)
	elif action == ACT_IWDQSPEC:
		btn.SetBAM ("stonspec", 0, 0)
		if GameCheck.IsIWD2 () and i > 3:
			tmp = i - 3
		else:
			tmp = 0
		state = SetQSpellBtn (btn, pc, tmp)
	elif action in [ACT_QSPELL1, ACT_QSPELL2, ACT_QSPELL3]:
		btn.SetBAM ("stonspel", 0, 0)
		tmp = action - ACT_QSPELL1
		state = SetQSpellBtn (btn, pc, tmp)
	elif action == ACT_IWDQITEM:
		if i > 3:
			tmp = (i + 1) % 3
		else:
			tmp = 0
		state = SetQSlotBtn (btn, pc, tmp, action)
	elif action == ACT_QSLOT1:
		tmp = 0
		state = SetQSlotBtn (btn, pc, tmp, action)
	elif action == ACT_QSLOT2:
		tmp = 1
		state = SetQSlotBtn (btn, pc, tmp, action)
	elif action == ACT_QSLOT3:
		tmp = 2
		state = SetQSlotBtn (btn, pc, tmp, action)
	elif action == ACT_QSLOT4:
		tmp = 3
		state = SetQSlotBtn (btn, pc, tmp, action)
	elif action == ACT_QSLOT5:
		tmp = 4
		state = SetQSlotBtn (btn, pc, tmp, action)

	return state



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
#character generation, alignment (GUICG3)
import GemRB
from GUIDefines import *
from ie_stats import *
import CharGenCommon
import GUICommon
import CommonTables


AlignmentWindow = 0
TextAreaControl = 0
DoneButton = 0

def OnLoad():
	global AlignmentWindow, TextAreaControl, DoneButton

	MyChar = GemRB.GetVar ("Slot")

	GemRB.SetVar("Alignment", None)
	
	KitName = GUICommon.GetClassRowName (MyChar)

	AlignmentOk = GemRB.LoadTable("ALIGNMNT")

	AlignmentWindow = GemRB.LoadWindow(3, "GUICG")

	# This section enables or disables different alignment selections
	# based on Class, and depends on the ALIGNMNT.2DA table
	#
	# For now, we just enable all buttons
	for i in range(9):
		Button = AlignmentWindow.GetControl(i+2)
		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
		Button.SetText (CommonTables.Aligns.GetValue (i,0))
		if AlignmentOk.GetValue (KitName, CommonTables.Aligns.GetValue (i, 4)) != 0:
			Button.SetState(IE_GUI_BUTTON_ENABLED)
		else:
			Button.SetState(IE_GUI_BUTTON_DISABLED)
		Button.OnPress (AlignmentPress)
		Button.SetVarAssoc("Alignment", i)

	BackButton = AlignmentWindow.GetControl(13)
	BackButton.SetText(15416)
	DoneButton = AlignmentWindow.GetControl(0)
	DoneButton.SetText(11973)
	DoneButton.MakeDefault()


	TextAreaControl = AlignmentWindow.GetControl(11)
	TextAreaControl.SetText(9602)

	DoneButton.OnPress (NextPress)
	BackButton.OnPress (lambda: CharGenCommon.back(AlignmentWindow))
	DoneButton.SetState(IE_GUI_BUTTON_DISABLED)
	AlignmentWindow.ShowModal(MODAL_SHADOW_GRAY)
	return

def AlignmentPress():
	Alignment = GemRB.GetVar("Alignment")
	TextAreaControl.SetText (CommonTables.Aligns.GetValue (Alignment,1))
	DoneButton.SetState(IE_GUI_BUTTON_ENABLED)
	GemRB.SetVar ("Alignment", CommonTables.Aligns.GetValue (Alignment,3))
	return

def NextPress():
	AlignmentWindow.Close()
	MyChar = GemRB.GetVar ("Slot")
	Alignment = GemRB.GetVar ("Alignment")
	GemRB.SetPlayerStat (MyChar, IE_ALIGNMENT, Alignment)
	CharGenCommon.next()



# GemRB - Infinity Engine Emulator
# Copyright (C) 2011 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# a listing of iwd2 feat indices

# KEEP SYNCHRONISED WITH gemrb/includes/ie_feats.h !!!
FEAT_AEGIS_OF_RIME = 0
FEAT_AMBIDEXTERITY = 1
FEAT_AQUA_MORTIS = 2
FEAT_ARMOUR_PROFICIENCY = 3
FEAT_ARMORED_ARCANA = 4
FEAT_ARTERIAL_STRIKE = 5
FEAT_BLIND_FIGHT = 6
FEAT_BULLHEADED = 7
FEAT_CLEAVE = 8
FEAT_COMBAT_CASTING = 9
FEAT_COURTEOUS_MAGOCRACY = 10
FEAT_CRIPPLING_STRIKE = 11
FEAT_DASH = 12
FEAT_DEFLECT_ARROWS = 13
FEAT_DIRTY_FIGHTING = 14
FEAT_DISCIPLINE = 15
FEAT_DODGE = 16
FEAT_ENVENOM_WEAPON = 17
FEAT_EXOTIC_BASTARD = 18
FEAT_EXPERTISE = 19
FEAT_EXTRA_RAGE = 20
FEAT_EXTRA_SHAPESHIFTING = 21
FEAT_EXTRA_SMITING = 22
FEAT_EXTRA_TURNING = 23
FEAT_FIENDSLAYER = 24
FEAT_FORESTER = 25
FEAT_GREAT_FORTITUDE = 26
FEAT_HAMSTRING = 27
FEAT_HERETICS_BANE = 28
FEAT_HEROIC_INSPIRATION = 29
FEAT_IMPROVED_CRITICAL = 30
FEAT_IMPROVED_EVASION = 31
FEAT_IMPROVED_INITIATIVE = 32
FEAT_IMPROVED_TURNING = 33
FEAT_IRON_WILL = 34
FEAT_LIGHTNING_REFLEXES = 35
FEAT_LINGERING_SONG = 36
FEAT_LUCK_OF_HEROES = 37
FEAT_MARTIAL_AXE = 38
FEAT_MARTIAL_BOW = 39
FEAT_MARTIAL_FLAIL = 40
FEAT_MARTIAL_GREATSWORD = 41
FEAT_MARTIAL_HAMMER = 42
FEAT_MARTIAL_LARGESWORD = 43
FEAT_MARTIAL_POLEARM = 44
FEAT_MAXIMIZED_ATTACKS = 45
FEAT_MERCANTILE_BACKGROUND = 46
FEAT_POWER_ATTACK = 47
FEAT_PRECISE_SHOT = 48
FEAT_RAPID_SHOT = 49
FEAT_RESIST_POISON = 50
FEAT_SCION_OF_STORMS = 51
FEAT_SHIELD_PROF = 52
FEAT_SIMPLE_CROSSBOW = 53
FEAT_SIMPLE_MACE = 54
FEAT_SIMPLE_MISSILE = 55
FEAT_SIMPLE_QUARTERSTAFF = 56
FEAT_SIMPLE_SMALLBLADE = 57
FEAT_SLIPPERY_MIND = 58
FEAT_SNAKE_BLOOD = 59
FEAT_SPELL_FOCUS_ENCHANTMENT = 60
FEAT_SPELL_FOCUS_EVOCATION = 61
FEAT_SPELL_FOCUS_NECROMANCY = 62
FEAT_SPELL_FOCUS_TRANSMUTE = 63
FEAT_SPELL_PENETRATION = 64
FEAT_SPIRIT_OF_FLAME = 65
FEAT_STRONG_BACK = 66
FEAT_STUNNING_FIST = 67
FEAT_SUBVOCAL_CASTING = 68
FEAT_TOUGHNESS = 69
FEAT_TWO_WEAPON_FIGHTING = 70
FEAT_WEAPON_FINESSE = 71
FEAT_WILDSHAPE_BOAR = 72
FEAT_WILDSHAPE_PANTHER = 73
FEAT_WILDSHAPE_SHAMBLER = 74
